/*
 *  C64_GP32.i by Mike Dawson, adapted from:
 *  C64_x.i - Put the pieces together, X specific stuff
 *
 *  Frodo (C) 1994-1997,2002 Christian Bauer
 *  Unix stuff by Bernd Schmidt/Lutz Vieweg
 */

#include "main.h"
extern "C" {
#include "menu.h"
#include "ui.h"
#include "input.h"
#include "gpmisc.h"
}

static int time_start=0;
int total_frames=0;

int current_joystick=1;

#ifndef HAVE_USLEEP

int usleep(unsigned long int microSeconds)
{
	//static int delay=0;
	//if(microSeconds<=1000) return 0;
	//microSeconds-=1000;
	int timeleft;

	int start_time=clock();
	float clockticks=((float)(microSeconds*CLOCKS_PER_SEC))/1000000;
	//if(delay>100) {
	//	printf("waiting %d %d\n", microSeconds, (int)clockticks);
	//	delay=0;
	//} else {
	//	delay++;
	//}

	//while(clock()<(start_time+(int)clockticks)) {
	while((timeleft=(start_time+(int)clockticks)-clock())>0) {
		if(timeleft>(CLOCKS_PER_SEC/50)) {
			waitline(1);
		}
	}
}
#endif


/*
 *  Constructor, system-dependent things
 */

void C64::c64_ctor1(void)
{
}

void C64::c64_ctor2(void)
{
}


/*
 *  Destructor, system-dependent things
 */

void C64::c64_dtor(void)
{
}


/*
 *  Start main emulation thread
 */

void C64::Run(void)
{
	// Reset chips
	TheCPU->Reset();
	TheSID->Reset();
	TheCIA1->Reset();
	TheCIA2->Reset();
	TheCPU1541->Reset();

	// Patch kernal IEC routines
	orig_kernal_1d84 = Kernal[0x1d84];
	orig_kernal_1d85 = Kernal[0x1d85];
	PatchKernal(ThePrefs.FastReset, ThePrefs.Emul1541Proc);

	quit_thyself = false;
	thread_func();
}


/*
 *  Vertical blank: Poll keyboard and joysticks, update window
 */

void C64::VBlank(bool draw_frame)
{
	double elapsed_time, speed_index;

	poll_input();
	kbd_buf_update(this);

	TheDisplay->PollKeyboard(TheCIA1->KeyMatrix, 
				TheCIA1->RevMatrix, &joykey);

	TheCIA1->Joystick1 = poll_joystick(0);
	TheCIA1->Joystick2 = poll_joystick(1);

	if(draw_frame) {
		TheDisplay->Update();
		if(keyboard_enabled||options_enabled||status_enabled)
			draw_ui(this);
		TheDisplay->BufSwap();

		// calculate time between vblanks
		int time=clock()-time_start;
		elapsed_time=(double)time*(1000000/CLOCKS_PER_SEC);
		speed_index=20000/(elapsed_time+1)*ThePrefs.SkipFrames*100;
		time_start=clock();

		if((speed_index>100) && ThePrefs.LimitSpeed) {
			usleep((unsigned long)(ThePrefs.SkipFrames*20000-elapsed_time));
			speed_index=100;
		}

		TheDisplay->Speedometer((int)speed_index);

		// calculate fps
		total_frames++;
		emu_fps=total_frames/(((clock()+1)/CLOCKS_PER_SEC)+1);
	}

}


/*
 *  Open/close joystick drivers given old and new state of
 *  joystick preferences
 */

void C64::open_close_joysticks(bool oldjoy1, bool oldjoy2, bool newjoy1, bool newjoy2)
{
}


/*
 *  Poll joystick port, return CIA mask
 */

uint8 C64::poll_joystick(int port)
{
	uint8 j = 0xff;

	if(port!=current_joystick) return j;

	if(options_enabled||keyboard_enabled) return j;

	if(button_state&LEFT_PRESSED) j&=0xfb;
	if(button_state&RIGHT_PRESSED) j&=0xf7;
	if(button_state&UP_PRESSED) j&=0xfe;
	if(button_state&DOWN_PRESSED) j&=0xfd;
	if(button_state&A_PRESSED) j&=0xef;
	return j;
}


/*
 * The emulation's main loop
 */

void C64::thread_func(void)
{
	int linecnt = 0;

#ifdef FRODO_SC
	while (!quit_thyself) {

		// The order of calls is important here
		if (TheVIC->EmulateCycle())
			TheSID->EmulateLine();
		TheCIA1->CheckIRQs();
		TheCIA2->CheckIRQs();
		TheCIA1->EmulateCycle();
		TheCIA2->EmulateCycle();
		TheCPU->EmulateCycle();

		if (ThePrefs.Emul1541Proc) {
			TheCPU1541->CountVIATimers(1);
			if (!TheCPU1541->Idle)
				TheCPU1541->EmulateCycle();
		}
		CycleCounter++;
#else
	while (!quit_thyself) {

		// The order of calls is important here
		int cycles = TheVIC->EmulateLine();
		TheSID->EmulateLine();
#if !PRECISE_CIA_CYCLES
		TheCIA1->EmulateLine(ThePrefs.CIACycles);
		TheCIA2->EmulateLine(ThePrefs.CIACycles);
#endif

		if (ThePrefs.Emul1541Proc) {
			int cycles_1541 = ThePrefs.FloppyCycles;
			TheCPU1541->CountVIATimers(cycles_1541);

			if (!TheCPU1541->Idle) {
				// 1541 processor active, alternately execute
				//  6502 and 6510 instructions until both have
				//  used up their cycles
				while (cycles >= 0 || cycles_1541 >= 0)
					if (cycles > cycles_1541)
						cycles -= TheCPU->EmulateLine(1);
					else
						cycles_1541 -= TheCPU1541->EmulateLine(1);
			} else
				TheCPU->EmulateLine(cycles);
		} else
			// 1541 processor disabled, only emulate 6510
			TheCPU->EmulateLine(cycles);
#endif
		linecnt++;
	}
}

void C64::Pause() {
	have_a_break=true;
	TheSID->PauseSound();
}

void C64::Resume() {
	have_a_break=false;
	TheSID->ResumeSound();
}

