/*
 *  Display_GP32.i by Mike Dawson, adapted from:
 *  Display_x.i - C64 graphics display, emulator window handling,
 *                X specific stuff
 *
 *  Frodo (C) 1994-1997,2002 Christian Bauer
 *  X11 stuff by Bernd Schmidt/Lutz Vieweg
 */

#include "SAM.h"
#include "C64.h"

extern "C" {
#include "pogo.h"
#include "gpmisc.h"
#include "input.h"
#include "ui.h"
#include "menu.h"
}

#define GP32dr

extern uint16 *screen;

uint8 *bufmem;

uint8 *emu_screen;
uint8 *emu_buffers[2];
int emu_buf=0;

static int keystate[256];


/*
  C64 keyboard matrix:

    Bit 7   6   5   4   3   2   1   0
  0    CUD  F5  F3  F1  F7 CLR RET DEL
  1    SHL  E   S   Z   4   A   W   3
  2     X   T   F   C   6   D   R   5
  3     V   U   H   B   8   G   Y   7
  4     N   O   K   M   0   J   I   9
  5     ,   @   :   .   -   L   P   +
  6     /   ^   =  SHR HOM  ;   *   
  7    R/S  Q   C= SPC  2  CTL  <-  1
*/

#define MATRIX(a,b) (((a) << 3) | (b))




/*
 *  Display constructor: Draw Speedometer/LEDs in window
 */

C64Display::C64Display(C64 *the_c64) : TheC64(the_c64)
{

}


/*
 *  Display destructor
 */

C64Display::~C64Display()
{
}


/*
 *  Prefs may have changed
 */

void C64Display::NewPrefs(Prefs *prefs)
{
}


/*
 *  Connect to X server and open window
 */

int init_graphics(void)
{
	int i,j;

	// gfx buffer
#ifndef GP32dr
	bufmem=(uint8 *)malloc((DISPLAY_X*DISPLAY_Y)+4096);
	bufmem=(uint8 *)(((uint32)bufmem+4095)&0xfffff000);
#endif
	//mmu_change(bufmem, ((uchar*)bufmem)+DISPLAY_X*DISPLAY_Y-1, 0xff2);

	// 8bpp display memory
	for(j=0; j<2; j++) {
		//emu_screen=(uint8 *)malloc(320*240+4096);
		emu_screen=(uint8 *)malloc((DISPLAY_X*DISPLAY_Y)+4096);
		emu_screen=(uint8 *)(((uint32)emu_screen+4095)&0xfffff000);
		//mmu_change(emu_screen, ((uchar*)emu_screen)+(DISPLAY_X*DISPLAY_Y)-1, 0xff2);
		for(i=0; i<(DISPLAY_X*DISPLAY_Y); i++) *(emu_screen+i)=0x0;
		emu_buffers[j]=emu_screen;
	}

	switch_screen(emu_buffers[0]);
	emu_screen=double_buffer(120, 320, emu_buffers, &emu_buf);
#ifdef GP32dr
	bufmem=emu_screen;
#endif

	for(i=0; i<256; i++) keystate[i]=0;
}


/*
 *  Redraw bitmap
 */

void C64Display::Update(void)
{
	drive8active=led_state[0];

	//copy from bufmem to emu_screen
	// todo: have emu core render straight to display

#ifndef GP32dr
	// jeff fs fast blit routine
	XlatVGABuffer(bufmem, emu_screen+(240*32));
#endif
}

void C64Display::BufSwap(void)
{
	emu_screen=double_buffer(120, 320, emu_buffers, &emu_buf);
#ifdef GP32dr
	bufmem=emu_screen;
#endif
}


/*
 *  Draw one drive LED
 */

void C64Display::draw_led(int num, int state)
{

}


/*
 *  LED error blink
 */

void C64Display::pulse_handler(...)
{

}


/*
 *  Draw speedometer
 */

void C64Display::Speedometer(int speed)
{
	emu_speed=speed;
	static int delay=0;
	if(delay>=20) {
		emu_minutes=(clock()/CLOCKS_PER_SEC)/60;
		emu_seconds=(clock()/CLOCKS_PER_SEC)%60;
		delay=0;
	} else {
		delay++;
	}
}


/*
 *  Return pointer to bitmap data
 */

uint8 *C64Display::BitmapBase(void)
{
#ifdef GP32dr
	return bufmem+(240*32)+239;
#else
	return bufmem;
#endif
}


/*
 *  Return number of bytes per row
 */

int C64Display::BitmapXMod(void)
{
	return DISPLAY_X;
}

void KeyPress(int key, uint8 *key_matrix, uint8 *rev_matrix) {
	int c64_byte, c64_bit, shifted;
	if(!keystate[key]) {
		keystate[key]=1;
		c64_byte=key>>3;
		c64_bit=key&7;
		shifted=key&128;
		c64_byte&=7;
		if(shifted) {
			key_matrix[6] &= 0xef;
			rev_matrix[4] &= 0xbf;
		}
		key_matrix[c64_byte]&=~(1<<c64_bit);
		rev_matrix[c64_bit]&=~(1<<c64_byte);
	}
}

void KeyRelease(int key, uint8 *key_matrix, uint8 *rev_matrix) {
	int c64_byte, c64_bit, shifted;
	if(keystate[key]) {
		keystate[key]=0;
		c64_byte=key>>3;
		c64_bit=key&7;
		shifted=key&128;
		c64_byte&=7;
		if(shifted) {
			key_matrix[6] |= 0x10;
			rev_matrix[4] |= 0x40;
		}
		key_matrix[c64_byte]|=(1<<c64_bit);
		rev_matrix[c64_bit]|=(1<<c64_byte);
	}
}

/*
 *  Poll the keyboard
 */

void C64Display::PollKeyboard(uint8 *key_matrix, uint8 *rev_matrix, uint8 *joystick)
{
	int key;

	// b=space
	key=MATRIX(7,4);
	if(button_state&B_PRESSED) {
		KeyPress(key, key_matrix, rev_matrix);
	} else {
		KeyRelease(key, key_matrix, rev_matrix);
	}

	// select=shift/run-stop
	key=MATRIX(7,7);
	key|=128;
	if(button_state&SELECT_PRESSED) {
		KeyPress(key, key_matrix, rev_matrix);
	} else {
		KeyRelease(key, key_matrix, rev_matrix);
	}

	// check virtual keyboard
	if(keyboard_enabled) {
		if(vkey_pressed) {
			KeyPress(vkey, key_matrix, rev_matrix);
			vkey_pressed=0;
		}
		if(vkey_released) {
			KeyRelease(vkey, key_matrix, rev_matrix);
			vkey_released=0;
			//vkey=0;
		}
	}
}


/*
 *  Check if NumLock is down (for switching the joystick keyboard emulation)
 */

bool C64Display::NumLock(void)
{
}


/*
 *  Allocate C64 colors
 */

void C64Display::InitColors(uint8 *colors)
{
	int i;

	// set up 8bpp mode palette
	palette_r=(uint8 *)palette_red;
	palette_g=(uint8 *)palette_green;
	palette_b=(uint8 *)palette_blue;
	ui_set_palette();

	// frodo internal 8 bit palette
	for(i=0; i<256; i++) {
		colors[i] = i & 0x0f;
	}
}


/*
 *  Show a requester (error message)
 */

long int ShowRequester(char *a,char *b,char *)
{
	printf("%s: %s\n", a, b);
	return 1;
}

