/**
 * @file sid_epoc.i - 6581 emulation, EPOC specific stuff
 *
 * (c) 2001-2002 Alfred E. Heggestad
 */

//#ifndef __ER6__
//#include "alaw.h"
//#endif

#include "gp32/sound/gp32.h"
#include "gp32/sound/iis.h"

extern "C" {
#include "gpmisc.h"
}

// #define PCM_DUMP

/**
 * Buffer size: 2^9 == 512 bytes. Note that too large buffers will not work
 * very well: The speed of the C64 is slowed down to an average speed of
 * 100% by the blocking write() call in EmulateLine(). If you use a buffer
 * of, say 4096 bytes, that will happen only about every 4 frames, which
 * means that the emulation runs much faster in some frames, and much
 * slower in others.
 * On really fast machines, it might make sense to use an even smaller
 * buffer size.
 */
const int KDefaultBufSize = 256;
const int KDefaultBuffers = 16;

//#define smpbytes (770048)
//#define smpwords (smpbytes/4)

static int16 *smpstart;
//static unsigned char *smpend;
//static unsigned char *smpptr;
static volatile int playbuffer;
static volatile int writebuffer;

static unsigned long waitbuff[KDefaultBufSize];

//void DMA2_Done(void) __attribute__ ((interrupt ("IRQ")));
void DMA2_Done(void) __attribute__ ((naked));

void DMA2_Done(void) {
//void __attribute__((naked)) DMA2_Done(void) {
//void __attribute__((interrupt("IRQ"))) DMA2_Done(void) {
	unsigned long size;
	int nextbuffer;

	asm volatile ("STMDB    r13!,{r0-r12,lr}\n");
        /* invert color */
	// *(volatile unsigned long*)0x14A00418^=0xffffffff;

	nextbuffer=playbuffer+1;
	if(nextbuffer==KDefaultBuffers)nextbuffer=0;

	//return;

	if(nextbuffer==writebuffer) { // test buffer underrun
       		//GpPcmPlay((short unsigned int*)&waitbuff[0],KDefaultBufSize/2,0);
       		GpPcmPlay((short unsigned int*)&waitbuff[0],KDefaultBufSize*sizeof(unsigned long),0);
	} else {
       		//GpPcmPlay((short unsigned int*)&smpstart[(playbuffer*KDefaultBufSize)],KDefaultBufSize*2,1);
       		GpPcmPlay((short unsigned int*)&smpstart[(playbuffer*KDefaultBufSize)],KDefaultBufSize*sizeof(int16),1);
		playbuffer=nextbuffer;
	}

    	rSRCPND=BIT_DMA2;   //clear pending bit
    	rINTPND=BIT_DMA2;
    	//rINTPND;

	asm volatile (
                "LDMIA    r13!,{r0-r12,lr}\n"
                "SUBS PC,lr,#4\n"
                );

}

void soundstart(int16 *buffer,uint32 len) {

	/* initialize uda1330a by sending appropriate commands over L3 bus
       fsflag  00  : 512,384,256fs         (SC : System Clock Freq)
       type   000,  : iis,lsb16,lsb18,lsb20,msb                        */
	Init1330(0,0);

	/* setup IIS bus
       bits                  8 / 16 bit per channel
       type                  iis compatible samples / MSB samples
       mclk                  master clock    256,384fs
       bclk                  serial bitclock 16,32,48fs
       prescale a,b
        - higher numbers for a and b mean slower playing sample

	mclk at default cpu speed:
		mono:		stereo:   0
	bclk at default cpu speed:
		mono:		stereo:   1

	16bit, 44khz prescaler at default cpu speed:
		mono:		stereo:   2,2
	                at 99 mhz
		mono:		stereo:   8,7
    */
	//InitIIS(1,0, 0,1, 8,8);
	//InitIIS(1,0, 0,1, 12,12); // 100mhz
	InitIIS(1,0, 0,1, 16,16); // 133 mhz

//    GpPcmInit(PCM_S44,PCM_16BIT);

//    GpPcmInit(PCM_M44,PCM_8BIT);
//    GpPcmInit(PCM_M44,PCM_8BIT);
//    GpPcmInit(PCM_S44,PCM_16BIT);
//	GpPcmInit(PCM_M44,PCM_16BIT);

	ARMDisableInterrupt();

        //rINTMSK=0xffffffff;     // all irqs masked
        rINTMSK    |=(BIT_DMA2);
        swi_install_irq(nISR_DMA2,(void*)&DMA2_Done);

        /* irq regs */
        //rSRCPND    |=BIT_DMA2;
        //rINTPND    |=BIT_DMA2;
        //rINTMOD    &=~(BIT_DMA2);
        //rINTMSK    &=~(BIT_DMA2);

	ARMEnableInterrupt();

        /* setup pointers for playnextchunk() */
        smpstart=buffer;
//        smpend=smpstart+len;
//        smpptr=smpstart;
        /* start playing */
//        playnextchunk();
	//printf("calling gppcmplay %x %d\n", &waitbuff[0], KDefaultBufSize/2);
	GpPcmPlay((short unsigned int*)&waitbuff[0],KDefaultBufSize/2,1);

	playbuffer=1;
	writebuffer=0;
}


void DigitalRenderer::init_sound()
/**
 *  Initialization
 */
	{
    divisor = to_output = buffer_pos = 0;
    ready = false; // asume the worst

	sound_buffer = new int16[KDefaultBufSize*KDefaultBuffers];
	if(!sound_buffer)
		{
		ELOG1(_L8("AUDIO: no memory left for audio buffers :(\n"));
		return;
		}
	ELOG1(_L8("AUDIO: the SID chip is ready to Rock'n'Roll...\n"));
    ready = true;

    	//mmu_change(sound_buffer, ((uchar*)sound_buffer)
	//		+(KDefaultBufSize*KDefaultBuffers*2), 0xff2);

	buffer_pos=0;buffer_num=0;
	//memset(sound_buffer,0,KDefaultBufSize*KDefaultBuffers*2);
	memset(sound_buffer,0,KDefaultBufSize*KDefaultBuffers*sizeof(int16));
	//memset(waitbuff,0,KDefaultBufSize/2);
	memset(waitbuff,0,KDefaultBufSize*sizeof(unsigned long));
	//soundstart(sound_buffer,KDefaultBufSize*KDefaultBuffers*sizeof*2);
	soundstart(sound_buffer,KDefaultBufSize*KDefaultBuffers*sizeof(int16));

	}

DigitalRenderer::~DigitalRenderer()
/**
 *  Destructor
 */
	{
	delete sound_buffer;
	// disable sound
	}


void DigitalRenderer::Pause(void)
/**
 *  Pause sound output (do nothing)
 */
	{
	}


void DigitalRenderer::Resume(void)
/**
 * Resume sound output (do nothing)
 */
	{
	}


void DigitalRenderer::EmulateLine(void) {
/**
 * Fill buffer, sample volume (for sampled voice)
 */
	if(!ready) return;

	sample_buf[sample_in_ptr] = volume;
	sample_in_ptr = (sample_in_ptr + 1) % SAMPLE_BUF_SIZE;

    /**
     * Now see how many samples have to be added for this line
     */
	divisor += SAMPLE_FREQ;
	while (divisor >= 0) {
		divisor -= TOTAL_RASTERS*SCREEN_FREQ;
		to_output++;
	}

    /**
     * Calculate the sound data only when we have enough to fill
     * the buffer entirely.
     */
//    if ((buffer_pos + to_output) >= KDefaultBufSize)
//		{
//		int datalen = KDefaultBufSize - buffer_pos;
//		to_output -= datalen;
//		calc_buffer(sound_buffer + buffer_pos, datalen*2);

//		buffer_pos = 0;
    if ((to_output) >= KDefaultBufSize)
		{
//		int datalen = KDefaultBufSize;
			int nextbuffer;
			nextbuffer=writebuffer+1;if(nextbuffer==KDefaultBuffers)nextbuffer=0;

			if(nextbuffer!=playbuffer){ // check buffer overrun
				to_output -= KDefaultBufSize;
  				buffer_pos = (writebuffer*KDefaultBufSize);
				calc_buffer(&sound_buffer[buffer_pos], KDefaultBufSize*2);
		//		playbuffer=buffer_num;
				writebuffer++;if(writebuffer==KDefaultBuffers)writebuffer=0;
			}
			//DMA2_Done();
		}

//
// this is how they do it on Linux:
//
//		write(devfd, sound_buffer, sndbufsize*2);
//		}

	}

//
// this code is moved here from FixPoint.i to avoid global data
//

/// define as global variable (4x512=2048 bytes)
//FixPoint SinTable[(1<<ldSINTAB)];

#ifdef USE_FIXPOINT_MATHS

#define FIXPOINT_SIN_COS_GENERIC \
  if (angle >= 3*(1<<ldSINTAB)) {return(-SinTable[(1<<(ldSINTAB+2)) - angle]);}\
  if (angle >= 2*(1<<ldSINTAB)) {return(-SinTable[angle - 2*(1<<ldSINTAB)]);}\
  if (angle >= (1<<ldSINTAB)) {return(SinTable[2*(1<<ldSINTAB) - angle]);}\
  return(SinTable[angle]);

// sin and cos: angle is fixpoint number 0 <= angle <= 2 (*PI)
inline FixPoint DigitalRenderer::fixsin(FixPoint x)
{
  int angle = x;

  angle = (angle >> (FIXPOINT_PREC - ldSINTAB - 1)) & ((1<<(ldSINTAB+2))-1);
  FIXPOINT_SIN_COS_GENERIC
}

inline FixPoint DigitalRenderer::fixcos(FixPoint x)
{
  int angle = x;

  // cos(x) = sin(x+PI/2)
  angle = (angle + (1<<(FIXPOINT_PREC-1)) >> (FIXPOINT_PREC - ldSINTAB - 1)) & ((1<<(ldSINTAB+2))-1);
  FIXPOINT_SIN_COS_GENERIC
}

inline void DigitalRenderer::InitFixSinTab(void)
{
 ELOG2(_L8("InitFixSinTab [SinTable=0x%08x] ..."), SinTable);
  int i;
  float step;

  for (i=0, step=0; i<(1<<ldSINTAB); i++, step+=0.5/(1<<ldSINTAB))
  {
    SinTable[i] = FixNo(sin(M_PI * step));
  }
  ELOG1(_L8("[  OK  ]\n"));
}

#endif


