// ELOG.H
//
// (c) 2000-2002 Alfred E. Heggestad
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License version 2 as
//    published by the Free Software Foundation.
//
#if !defined(ELOG_H__)
#define ELOG_H__

/**
 * @file elog.h
 *
 * defines the interface to file logging for EPOC programs
 */


#if defined(__linux__) && !defined(__SYMBIAN32__) //&& !defined(GP32)
#define _DEBUG
#define _L8(a) (a)
#endif

//
// some logging frontend macros
//

#if defined(_DEBUG)
#define CTOR(a)	 ELOG4(_L8("created object %s (%d bytes) at 0x%08x\n"), #a, sizeof(a), this );
#define DTOR(a)	 ELOG4(_L8("deleted object %s (%d bytes) at 0x%08x\n"), #a, sizeof(a), this );
#else
#define CTOR(a)
#define DTOR(a)
#endif


#if defined(_DEBUG)
#if defined (__SYMBIAN32__)
#define __CHECK_NULL(p) if(p == NULL) \
	{ \
	ELOG3(_L8("unexpected NULL pointer in line %d file %s"), __LINE__, __FILE__); \
	User::Panic(_L("NULL"), 0); \
	}
#elif defined (GP32)
#define __CHECK_NULL(p) if(p == NULL) \
	{ \
	ELOG3(_L8("unexpected NULL pointer in line %d file %s"), __LINE__, __FILE__); \
	for(;;){}; \
	}
#else
#define __CHECK_NULL(p)
#endif
#endif


#define BOOL_BIT(var,msk) ( (var & msk)==(msk) )


//
// normal debugging
//
#if defined(_DEBUG)

#if defined(__SYMBIAN32__)

#ifndef __E32DEF_H__
  #ifdef NULL
  #undef NULL
  #endif
#endif

#include <f32file.h>
#define ELOG(a)           a
#define ELOG1(a)          CEpocLogger::DoLog(a)
#define ELOG2(a, b)       CEpocLogger::DoLog(a, (b) )
#define ELOG3(a, b, c)    CEpocLogger::DoLog(a, b, c)
#define ELOG4(a, b, c, d) CEpocLogger::DoLog(a, b, c, d)

class CEpocLogger : public CBase
	{
public:
	static void DoLog(TRefByValue<const TDesC8> aFmt, ...);
	};

#else // __SYMBIAN32__

// normally linux

#ifdef GP32
						  // fixme ! "UpdateGp32Screen()" shouldnt be here
#define ELOG(a)           a
#define ELOG1(a)          {cprintf(a);UpdateGp32Screen();}
#define ELOG2(a, b)       {cprintf(a, b );UpdateGp32Screen();}
#define ELOG3(a, b, c)    {cprintf(a, b, c);UpdateGp32Screen();}
#define ELOG4(a, b, c, d) {cprintf(a, b, c, d);UpdateGp32Screen();}
#else
#define ELOG(a)           a
#define ELOG1(a)          printf(a)
#define ELOG2(a, b)       printf(a, b )
#define ELOG3(a, b, c)    printf(a, b, c)
#define ELOG4(a, b, c, d) printf(a, b, c, d)
#endif // GP32

#endif // __SYMBIAN32__



#else // no debugging

#define ELOG(a)
#define ELOG1(a)
#define ELOG2(a, b)
#define ELOG3(a, b, c)
#define ELOG4(a, b, c, d)

#endif // _DEBUG


#endif  // ELOG_H__
