
extern "C" {
#include "libpogo/pogo.h"
#include "libpogo/GP32.h"
#include "ui.h"
#include "gpmisc.h"
#include "input.h"
}

//#define DISPLAY_X 384
//#define DISPLAY_Y 272
#define DISPLAY_X 320
#define DISPLAY_Y 240

void gpAppExecute(uint32 *ptr, char *path) {
	asm volatile("swi #5\n");
}

void app_execute(char *path) {
	uint32 *p;
	struct stat app;
	uint32 size;
	FILE *appdata;
	int j;

	stat(path, &app);
	size=app.st_size;
	p=(uint32 *)malloc(size+4);
	p[0]=size/4;
	appdata=fopen(path, "r");
	j=fread(p+1, size, 1, appdata);
	fclose(appdata);

	gpAppExecute(p, NULL);
}

void gp32reboot() {
	asm volatile("swi #4\n");
}

int sscanf( const char *str, const char *format, ...) {
	printf("unimplemented function sscanf called\n");
}

int fscanf(FILE *stream, const char *format, ...) {
	printf("unimplemented function fscanf called\n");
}

int feof(FILE *stream) {
	int i;
	char c;

	i=fread(&c, 1, 1, stream);
	if(i==0) {
		return 1;
	} else {
		fseek(stream, -1L, SEEK_CUR);
		return 0;
	}
}

extern uint16 *screen;
extern uint8 *emu_screen;

void waitline(int line) {
	while (line!=((rLCDCON1 >> 18) & 0x1ff) );
}

void waithblank(void) {
	int current_line=(rLCDCON1>>18)&0x1ff;
	while(current_line==((rLCDCON1>>18)&0x1ff));
}

void switch_console() {
	LCDCON1 emu_lcdcon1;
	LCDCON5 emu_lcdcon5;
	LCDSADDR1 emu_lcdsaddr1;
	LCDSADDR2 emu_lcdsaddr2;
	LCDSADDR3 emu_lcdsaddr3;

	memset(&emu_lcdsaddr1, 0, sizeof(emu_lcdsaddr1));
	memset(&emu_lcdsaddr2, 0, sizeof(emu_lcdsaddr2));
	memset(&emu_lcdsaddr3, 0, sizeof(emu_lcdsaddr3));
	memset(&emu_lcdcon1, 0, sizeof(emu_lcdcon1));
	memset(&emu_lcdcon5, 0, sizeof(emu_lcdcon5));

	emu_lcdcon1.BPPMODE = 12;
	emu_lcdcon1.CLKVAL = 3;
	emu_lcdcon1.ENVID = 1;
	emu_lcdcon1.MMODE = 0;
	emu_lcdcon1.PNRMODE = 3;
	emu_lcdcon1.LINECNT = 0;

	emu_lcdcon5.ENLEND = 0;
	emu_lcdcon5.HSTATUS = 2;
	emu_lcdcon5.INVENDLINE = 0;
	emu_lcdcon5.INVVCLK = 1;
	emu_lcdcon5.INVVD = 0;
	emu_lcdcon5.INVVDEN = 0;
	emu_lcdcon5.INVVFRAME = 1;
	emu_lcdcon5.INVVLINE = 1;
	emu_lcdcon5.VSTATUS = 2;
	emu_lcdcon5.RESERVED4 = 0;
	emu_lcdcon5.BSWP = 0;
	emu_lcdcon5.HWSWP = 1;

	emu_lcdsaddr1.LCDBANK = (unsigned int)screen >> 22; // bank
	emu_lcdsaddr1.LCDBASEU = ((unsigned int)screen & 0x3FFFFF) >> 1; // begin offset (per 16 bits)
	emu_lcdsaddr2.LCDBASEL = emu_lcdsaddr1.LCDBASEU + (320*240); // end offset (per 16 bits)
	emu_lcdsaddr3.PAGEWIDTH = 240;

	emu_lcdsaddr3.OFFSIZE = 0;

	waitline(1);

	rLCDCON1 = *(unsigned int *)&emu_lcdcon1;
	rLCDSADDR1 = *(unsigned int *)&emu_lcdsaddr1;
	rLCDSADDR2 = *(unsigned int *)&emu_lcdsaddr2;
	rLCDSADDR3 = *(unsigned int *)&emu_lcdsaddr3;
	rLCDCON5 = *(unsigned int *)&emu_lcdcon5;

	while(!(button_state&B_PRESSED)) poll_input();
	switch_screen(emu_screen);
}

void switch_screen(uint8 *display) {
	LCDCON1 emu_lcdcon1;
	LCDCON5 emu_lcdcon5;
	LCDSADDR1 emu_lcdsaddr1;
	LCDSADDR2 emu_lcdsaddr2;
	LCDSADDR3 emu_lcdsaddr3;

	memset(&emu_lcdsaddr1, 0, sizeof(emu_lcdsaddr1));
	memset(&emu_lcdsaddr2, 0, sizeof(emu_lcdsaddr2));
	memset(&emu_lcdsaddr3, 0, sizeof(emu_lcdsaddr3));
	memset(&emu_lcdcon1, 0, sizeof(emu_lcdcon1));
	memset(&emu_lcdcon5, 0, sizeof(emu_lcdcon5));

	emu_lcdcon1.BPPMODE = 11;
	emu_lcdcon1.CLKVAL = 3;
	emu_lcdcon1.ENVID = 1;
	emu_lcdcon1.MMODE = 0;
	emu_lcdcon1.PNRMODE = 3;
	emu_lcdcon1.LINECNT = 0;

	emu_lcdcon5.ENLEND = 0;
	emu_lcdcon5.HSTATUS = 2;
	emu_lcdcon5.INVENDLINE = 0;
	emu_lcdcon5.INVVCLK = 1;
	emu_lcdcon5.INVVD = 0;
	emu_lcdcon5.INVVDEN = 0;
	emu_lcdcon5.INVVFRAME = 1;
	emu_lcdcon5.INVVLINE = 1;
	emu_lcdcon5.VSTATUS = 2;
	emu_lcdcon5.RESERVED4 = 0;
	emu_lcdcon5.BSWP = 1;
	emu_lcdcon5.HWSWP = 0;

	emu_lcdsaddr1.LCDBANK = (unsigned int)display >> 22; // bank
	emu_lcdsaddr1.LCDBASEU = ((unsigned int)display & 0x3FFFFF) >> 1; // begin offset (per 16 bits)
	emu_lcdsaddr2.LCDBASEL = emu_lcdsaddr1.LCDBASEU + (320*240)/2; // end offset (per 16 bits)

	emu_lcdsaddr3.PAGEWIDTH = 120;
	emu_lcdsaddr3.OFFSIZE = 0;

	waitline(1);

	rLCDCON1 = *(unsigned int *)&emu_lcdcon1;
	rLCDSADDR1 = *(unsigned int *)&emu_lcdsaddr1;
	rLCDSADDR2 = *(unsigned int *)&emu_lcdsaddr2;
	rLCDSADDR3 = *(unsigned int *)&emu_lcdsaddr3;
	rLCDCON5 = *(unsigned int *)&emu_lcdcon5;

}

uint8 *double_buffer(int x, int y, uint8 *buffers[], int *sw) {
	uint32 ptr=(uint32)buffers[*sw];
	ptr+=(240*32);
	(*sw)^=1;

	if(options_enabled||keyboard_enabled) {
		waitline(1);
	} else {
		waithblank();
	}

	rLCDSADDR1=((ptr&0x0fc00000)>>1)|((ptr&0x3ffffe)>>1);
	rLCDSADDR2=((ptr&0x003ffffe)>>1)+(x*y);
	return buffers[*sw];
}


