
extern "C" {
#include "libpogo/pogo.h"
#include "input.h"
#include "ui.h"
}

#include "C64.h"
#include "Prefs.h"

uint32 button_state;
extern int current_joystick;

extern void reboot_gp32(void);

void poll_input(void) {
	static uint32 last_pbdat=0;
	static uint32 last_pedat=0;
	uint32 current_pbdat;
	uint32 current_pedat;

	if(!last_pbdat) last_pbdat=(*(volatile uint32 *)0x1560000c);
	if(!last_pedat) last_pedat=(*(volatile uint32 *)0x15600030);

	current_pbdat=(*(volatile uint32 *)0x1560000c);
	current_pbdat&=0x0000ff00;
	current_pedat=(*(volatile uint32 *)0x15600030);
	current_pedat&=0x000000c0;

	if(current_pbdat!=last_pbdat) {
		/* left */
		if((current_pbdat^last_pbdat)&0x00000100) {
			if(current_pbdat&0x00000100) {
				button_state&=~(uint32)LEFT_PRESSED;
			} else {
				button_state|=LEFT_PRESSED;
			}
		}
		/* right */
		if((current_pbdat^last_pbdat)&0x00000400) {
			if(current_pbdat&0x00000400) {
				button_state&=~(uint32)RIGHT_PRESSED;
			} else {
				button_state|=RIGHT_PRESSED;
			}
		}
		/* up */
		if((current_pbdat^last_pbdat)&0x00000800) {
			if(current_pbdat&0x00000800) {
				button_state&=~(uint32)UP_PRESSED;
			} else {
				button_state|=UP_PRESSED;
			}
		}
		/* down */
		if((current_pbdat^last_pbdat)&0x00000200) {
			if(current_pbdat&0x00000200) {
				button_state&=~(uint32)DOWN_PRESSED;
			} else {
				button_state|=DOWN_PRESSED;
			}
		}
		/* left shoulder button */
		if((current_pbdat^last_pbdat)&0x00001000) {
			if(current_pbdat&0x00001000) {
				button_state&=~(uint32)LSHOULDER_PRESSED;
			} else {
				button_state|=LSHOULDER_PRESSED;
				if(!keyboard_enabled) keyboard_enabled=1;
				else keyboard_enabled=0;
			}
		}
		/* button B */
		if((current_pbdat^last_pbdat)&0x00002000) {
			if(current_pbdat&0x00002000) {
				button_state&=~(uint32)B_PRESSED;
			} else {
				button_state|=B_PRESSED;
			}
		}
		/* button A */
		if((current_pbdat^last_pbdat)&0x00004000) {
			if(current_pbdat&0x00004000) {
				button_state&=~(uint32)A_PRESSED;
			} else {
				button_state|=A_PRESSED;
			}
		}
		/* right shoulder button */
		if((current_pbdat^last_pbdat)&0x00008000) {
			if(current_pbdat&0x00008000) {
				button_state&=~(uint32)RSHOULDER_PRESSED;
			} else {
				button_state|=RSHOULDER_PRESSED;
				if(!status_enabled) status_enabled=1;
				else status_enabled=0;
			}
		}
		last_pbdat=current_pbdat;
	}

	if(current_pedat!=last_pedat) {
		/* start */
		if((current_pedat^last_pedat)&0x00000040) {
			if(current_pedat&0x00000040) {
				button_state&=~(uint32)START_PRESSED;
			} else {
				button_state|=START_PRESSED;
				if(!options_enabled) options_enabled=1;
				else options_enabled=0;
			}
		} 
		/* select */
		if((current_pedat^last_pedat)&0x00000080) {
			if(current_pedat&0x00000080) {
				button_state&=~(uint32)SELECT_PRESSED;
			} else {
				button_state|=SELECT_PRESSED;
			}
		}
		last_pedat=current_pedat;
	}
}

char kbd_feedbuf[255];
int kbd_feedbuf_pos;

void kbd_buf_feed(char *s) {
	strcpy(kbd_feedbuf, s);
	kbd_feedbuf_pos=0;
}

void kbd_buf_update(C64 *TheC64) {
	if((kbd_feedbuf[kbd_feedbuf_pos]!=0)
			&& TheC64->RAM[198]==0) {
		TheC64->RAM[631]=kbd_feedbuf[kbd_feedbuf_pos];
		TheC64->RAM[198]=1;

		kbd_feedbuf_pos++;
	}
}



