/*
 * Text console device (console.c)
 * Part of libpogo, a c-library replacent for GBA
 * Programmed by Jonas Minnberg (Sasq)
 *
 * DESCRIPTION
 * Device for text output. Supports fixed and proportional fonts,
 * limited ANSI (colors, positioning, bold, scrolling etc).
 *
 **/

#include "core.h"
#include "font.h"
#include "device.h"
#include "screen.h"

#include "console.h"
#include "ffont.h"

#define TO16(x) (((x&0xF80000)>>8) | ((x&0x00F800)>>5) | ((x&0x0000F8)>>2))

uint32 colors[16] = 
{ 
	0x000000,
	0xC00000,
	0x00C000,
	0xC0C000,
	0x0000C0,
	//0x3E31A2,
	0xC000C0,
	0x00C0C0,
	//0x7E72DA,
	0xC0C0C0,

	0x000000,
	0xFF0000,
	0x00FF00,
	0xFFFF00,
	0x0000FF,
	0xFF00FF,
	0x00FFFF,
	0xFFFFFF,
};

typedef struct {
	Font *font;
//	uchar *pixels;
	uint16 width;
	uint16 height;
	int bufsize;
	int mode;
	int textpos;
	int x,y;
	int rborder;
	int bborder;
} Console;

static Font *fonts[4];

static int attrs;// = 0;

//static int fgcol;// = 0;
//static int bgcol; // = 0;

#define MAX_LINES 50
#define MAX_COLS 80

static char lines[MAX_LINES][MAX_COLS];

/* Points to one string for each line in buffer/screen */
static char *lineptrs[MAX_LINES];

/* Points to current char in current buffer */
char *lineptr;

static Console default_console;
static Console *con;


static Device condev;

static char *console_getline(int l)
{
	return lineptrs[l];
}

int scroll_start = 0;
int scroll_stop = 20;

extern int16 *screen;

static void console_scroll(int start, int rows, int count)
{
	int i,j;
	for(j=0; j<320; j++) {
		for(i=239; i>7; i--) {
			*(screen+(j*240)+i)=*(screen+(j*240)+(i-8));
		}
	}
	for(j=0; j<320; j++) {
		for(i=7; i>=0; i--) {
			*(screen+(j*240)+i)=0xfffe;
		}
	}
/*	int size;
	char *str;
	uint16 col = bgcol | (bgcol<<8);

	int i,fh = con->font->height;
	uint16 *d = (uint16 *)&con->pixels[start*fh*con->width];
	uint16 *s = (uint16 *)&con->pixels[(start+1)*fh*con->width];

	size = ((rows-1) * fh * con->width) / 2;

	while(size--)
		*d++ = *s++;
	size = (fh * con->width) /2;
	while(size--)
		*d++ = col;

	str = lineptrs[start];
	for(i=start; i<(start+rows)-1; i++) {
		if(lineptr == lineptrs[i+1])
			lineptr = lineptrs[i];
		lineptrs[i] = lineptrs[i+1];
	}
	if(lineptr == lineptrs[(start+rows)-1])
		lineptr = str;
	lineptrs[(start+rows)-1] = str;
	*str = 0;
*/

}

static void scroll_screen(void)
{
	console_scroll(scroll_start, scroll_stop-scroll_start, 1);
}

static void console_areaclear(int start, int rows)
{
	int i;

	screen_clear_bg(screen_offset(0, con->font->height * start), con->width, con->font->height * rows);

	for(i=start; i<(start+rows); i++)
		lineptrs[i][0] = 0;

}


static void clear_screen(void)
{
	int i;
	
	screen_clear_bg(screen_ptr(0), con->width, con->height);

	for(i=0; i<MAX_LINES; i++)
	{
		lineptrs[i][0] = 0;
		lineptrs[i] = lines[i];
	}
}

static void console_realxy(int x, int y)
{
	int w = 0;

	lineptr = lineptrs[y];

	while(x > w) {
		if(!(*lineptr)) { 
			*lineptr = ' ';
			lineptr[1] = 0;
		}
		w += font_putchar(con->font, *lineptr, NULL, con->width);
		lineptr++;
	}
	//x += (w-x);

	con->x = x;
	con->y = y;
}

/*
static void console_gotoxy(int x, int y)
{
	int w = 0;

	lineptr = lineptrs[y];

	while(x) {
		if(!(*lineptr)) {
			*lineptr = ' ';
			lineptr[1] = 0;
		}
		w += font_putchar(con->font, *lineptr, NULL, con->width);
		lineptr++;
		x--;
	}
	con->x = w;
	con->y = y;
}
*/

static void console_linekill(void)
{
	console_areaclear(con->y, 1);
	con->x = 0;
	lineptr = lineptrs[con->y];
	*lineptr = 0;
}

void console_setcolor(int fg, int bg)
{
	screen_set_fgbg(TO16(colors[fg]), TO16(colors[bg]));
}

#define isalpha(c) ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z'))
#define isdigit(c) ((c >= '0' && c <= '9') || (c == '-'))

static int atoi(const char *s)
{
	int v = 0;
	uchar neg = 0;

	while(*s == ' ')
		*s++;

	if(*s == '-') {
		s++;
		neg = 1;
	}

	while(isdigit(*s)) {
		v *= 10;
		v += (*s - 0x30);
		s++;
	}
	if(neg)
		v = -v;

	return v;
}

int parse_ansi(char *s, int *args)
{
	int i=0;
	if(*s == '[') s++;

	while(!isalpha(*s))
	{
		if(isdigit(*s))
		{
			args[i] = atoi(s);
			while(isdigit(*s))
				s++;
			continue;
		}
		if(*s == ';')
			i++;
		s++;
	}
	return i+1;

}

void handle_esc(char *s, char c)
{
	int args[4] = {0, 0, 0, 0};
	int w,i,l;
	int fgcol, bgcol;
	//uchar *ptr = &con->pixels[con->x + (con->y * con->font->height) * con->width];
	int dest = XY(con->x, con->y * con->font->height);

	switch(c)
	{
	case '(':
		con->font = fonts[0];
		break;
	case ')':
		con->font = fonts[1];
		break;
	case 'f':
		parse_ansi(s, args);
		con->font = fonts[args[0]];
		break;
	case 'K':
		w = con->x;
		while((w + con->font->charwidth) < con->width) {
			w += font_putchar(con->font, ' ', dest, con->width);
			dest += XY(con->font->charwidth, 0);
		}
		*lineptr = 0;
		break;
	case 'H':
		l = parse_ansi(s, args);
		console_realxy(args[1], args[0]);
		break;
	case 'A':
		args[0] = 1;
		parse_ansi(s, args);
		con->y -= args[0];
		break;
	case 'B':
		args[0] = 1;
		parse_ansi(s, args);
		con->y += args[0];
		break;
	case 'r':
		args[0] = 0;
		args[1] = con->bborder;
		parse_ansi(s, args);
		scroll_start = args[0];
		scroll_stop = args[1];
		break;
	case 'M':
		console_scroll(scroll_start, scroll_stop, 1);
		break;
	case 'm':
		l = parse_ansi(s, args);
		for(i=0; i<l; i++) {
			if(args[i] < 9)
				attrs = args[i];
			else
			if(args[i] < 38)
				fgcol = args[i] - 30;
			else
				bgcol = args[i] - 40;
		}

/*
		if(attrs == 7)
			font_setcolor(bgcol, fgcol + (attrs&2 ? 0 : 8));
		else*/
		//if(fgcol > 8)
		//	fgcol -= 8;
		if(attrs & 2)
			con->font->flags |= FFLG_BOLD;
		else
			con->font->flags &= (~FFLG_BOLD);

		console_setcolor((attrs & 4) ? fgcol + 8 : fgcol, bgcol);
	}
}

static uchar esc_mode = 0;
static char escstring[10];
static char *escptr;

static void console_putchar(char c)
{
	char *p;
	int w = 0;

	Font *font = con->font;
	//if(pos >= bufsize)
	//	scroll(con);
	//uchar *ptr = &con->pixels[con->x + (con->y * font->height) * con->width];
	int dest = XY(con->x, con->y * con->font->height);

	if(esc_mode) {
		*escptr++ = c;
		*escptr = 0;
		esc_mode--;
		if(isalpha(c)) {
			esc_mode = 0;
			handle_esc(escstring, c);
		}
		return;
	}

	switch(c) {
	case CHAR_LEFT:
		break;
	case CHAR_RIGHT:
		break;
	case 8:
		p = lineptrs[con->y];
		if(lineptr > p) {
			w = font_putchar(font, *(--lineptr), DONT_DRAW, con->width);
			*lineptr = 0;
			dest -= XY(w, 0);
			font_text(font, "  ", dest);
			w = -w;
		}
		break;
	case 12:
		w = 0;
		con->x = con->y = 0;
		clear_screen();
		break;
	case 27:
		esc_mode = 10;
		escptr = escstring;
		*escptr = 0;
		return;
	case 10:
	case 13:
		w = 0;
		con->x = 0;

		if((con->y + 1) >= con->bborder)
		{
			if(con->mode & CM_LINEWRAP)
				scroll_screen();
		}
		else
			con->y++;
		lineptr = lineptrs[con->y];
		*lineptr = 0;
		break;
	case 9:
		//w = font_text(font, "    ", NULL, con->width);
		w = font_text(font, "    ", dest);
		break;
	case 250:
		console_linekill();
		w = 0;
		break;
	default:
		//if(c < font->first || c > font->last)
		//	c = ' ';
		w = font_putchar(font, c, DONT_DRAW, con->width);
		if(w + con->x > con->rborder)
		{
			if(con->mode & CM_LINEWRAP)
			{
				if((con->y + 1) >= con->bborder)
					scroll_screen();
				else
					con->y++;
				con->x = 0;
				lineptr = lineptrs[con->y];
				*lineptr = 0;
				dest = XY(con->x, con->y * font->height);
			}
			else
				return;
		}
		if(w)
			*lineptr++ = c;
		*lineptr = 0;
		w = font_putchar(font, c, dest, con->width);
	}
	con->x += w;
}

static void console_setfont(Font *font, int num)
{
	if(font)
	{
		fonts[num] = font;
		con->font = font;
	} else
		con->font = fonts[num];
	con->bborder = con->height / font->height;
}

static int console_write(int fd, const void *dest, int size)
{
	int i;
	char *s = (char *)dest;
	for(i=0; i<size; i++)
		console_putchar(s[i]);

	return size;
}

static int console_read(int fd, void *dest, int size)
{
	return 0;
}

static int console_lseek(int fd, int offset, int origin)
{
	return -1;
}

void console_refresh(void)
{
	//uint32 *p = (uint32*)con->pixels;
	//char *ptr = con->pixels;
	int dest;
	//uint32 col = bgcol | (bgcol<<8) | (bgcol<<16) | (bgcol<<24);
	int i;
	//while(i--)
	//	*p++ = col;

	dest = XY(0,0);
	//ptr = con->pixels;
	for(i=0; i<con->bborder; i++)
	{
		font_text(con->font, lineptrs[i], dest);
		dest += XY(0, con->font->height);
	}
}
						 

static int console_open(const char *name, int mode)
{
	return 0;
}

static int console_ioctl(int fd, int req, va_list vl)
{
	char *s;

	switch(req) {
	case CC_CLEAR:
		console_areaclear(va_arg(vl, int), va_arg(vl, int));
		return 0;
	case IO_GETPARAM:
		switch(va_arg(vl, int))
		{
			case CP_TEXWIDTH:
				s = va_arg(vl, char*);
				return font_text(con->font, s, DONT_DRAW);
			case CP_WIDTH:
				return con->width;
			case CP_HEIGHT:
				return con->height;
			default:
				return -1;
		}
	case CC_GETWIDTH:
		s = va_arg(vl, char*);
		return font_text(con->font, s, DONT_DRAW);
	case IO_SETMODE:
		con->mode = va_arg(vl, int);
		return 0;
	case IO_GETMODE:
		return con->mode;
	case CC_SETFONT:
		console_setfont(va_arg(vl, Font*), va_arg(vl, int));
		return 0;
	case CC_GETFONT:
		return (int)fonts[va_arg(vl, int)];
	case CC_GETXY:
		return (con->x << 16) | con->y;
	case CC_GETLINE:
		return (int)console_getline(va_arg(vl, int));
	}
	return -1;
}

void console_init(void)
{
	//uchar *pixels;
	int width, height, fd;

	con = &default_console;

/*	fd = open("/dev/screen", 0);
	if(fd >= 0) {
		ioctl(fd, SC_SETMODE, 1);
		pixels = (char *)lseek(fd, 0, SEEK_MEM);
		width = ioctl(fd, IO_GETPARAM, SCREEN_WIDTH);
		height = ioctl(fd, IO_GETPARAM, SCREEN_HEIGHT);
	} else
		return;
	close(fd);
*/
	width = SCR_WIDTH;
	height = SCR_HEIGHT;

	fonts[0] = fonts[1] = con->font = font_memload(fixed_font);
	//con->pixels = pixels;
	con->x = 0;
	con->y = 0;
	con->textpos = 0;
	con->width = width;
	con->height = height;
	con->rborder = width;
	con->bborder = height / con->font->height;
	con->mode = 0;

	clear_screen();
	console_setcolor(0, 15);
	//fgcol = 7;

	lineptr = lineptrs[0];

	memset(&condev, 0, sizeof(condev));
	condev.open = console_open;
	condev.write = console_write;
	condev.ioctl = console_ioctl;
	condev.read = console_read;
	condev.lseek = console_lseek;

	device_register(&condev, "/dev/con", NULL, 1);
}
