#ifndef DEVICE_H
#define DEVICE_H

#include "core.h"

#ifdef __cplusplus
extern "C" {
#endif

#define MAX_PATH 256


typedef uint16 time_t;

typedef struct
{
	char name[32];
	int size;
	int start;

} __attribute__ ((packed)) Romfile;

struct stat 
{
  uchar		st_dev;
  uint16	st_ino;
  uint16	st_mode;
  uchar		st_nlink;
  uchar		st_uid;
  uchar		st_gid;
  uchar		st_rdev;
  uint32	st_size;
  time_t	st_atime;
  time_t	st_mtime;
  time_t	st_ctime; 
};

#define	S_IREAD		0x0000400	/* read permission, owner */
#define	S_IWRITE 	0x0000200	/* write permission, owner */
#define	S_IEXEC		0x0000100	/* execute/search permission, owner */
#define S_IFDIR     0x0004000 

#define S_ISDIR(m)  ((m) & S_IFDIR)

#define S_BLKSIZE  1024 /* size of a block */ 

typedef struct {
	int (*open)(const char *name, int flags);
	int (*read)(int fd, void *dest, int size);
	int (*write)(int fd, const void *dest, int size);
	int (*close)(int fd);
	int (*lseek)(int fd, int offset, int origin);
	int (*ioctl)(int fd, int request, va_list vlist);
	int (*stat)(const char *path, struct stat *buffer);
	int (*remove)(const char *name);
	int (*rename)(const char *oldname, const char *newname);
	int (*dread)(int fd, char *dest);

} Device;

int device_register(Device *dev, char *name, volatile void (*irqfunc)(void), int fd);

int open(const char *name, int flags);
int read(int fd, void *dest, int size);
int dread(int fd, char *dest);
int write(int fd, const void *dest, int size);
int close(int fd);
int lseek(int fd, int offset, int origin);
int tell(int fd);
int ioctl(int fd, int request, ...);
int remove(const char *name);
int stat(const char *name, struct stat *buffer);
int chdir(const char *path);

void execv(const char *cmdname, const char *const *argv);


#define	SEEK_SET	0	/* set file offset to offset */
#define	SEEK_CUR	1	/* set file offset to current plus offset */
#define	SEEK_END	2	/* set file offset to EOF plus offset */
#define SEEK_MEM    3   /* Return memory position if a romfile */

#define	O_READ		0x0001	/* read enabled */
#define	O_WRITE		0x0002	/* write enabled */
#define	O_RDWR		0x0002
#define	O_APPEND	0x0008	/* append (writes guaranteed at the end) */ 
#define	O_CREAT		0x0200	/* open with file create */
#define	O_TRUNC		0x0400	/* open with truncation */ 

#define O_RDONLY	0x0001

/* ioctl commands */

/* Standard ioctl commands */
#define IO_GETMODE 1
#define IO_SETMODE 2
#define IO_GETPARAM 3
#define IO_SETPARAM 4

/* Output console ioctl commands */
#define CC_GETWIDTH 5
#define CC_CLEAR 6
#define CC_SETFONT 9
#define CC_GETXY 10
#define CC_GETLINE 11
#define CC_GETFONT 13

/* Input console ioctl commands */
#define KC_SETIMAGE 9
#define KC_GETSCREENOFFSET 10
#define KC_ADDHISTORY 5
#define KC_SETDICT 6
#define KC_BINDKEY 7
#define KC_SETCOMPLETION 8

#define KC_REPEATMASK 11

/* Parameters */

#define CM_LINEWRAP 1
#define CM_SHOWCURSOR 2
#define CM_CURSORFOLLOW 4

#define KM_RAW 1
#define KM_SHOWKBD 2
#define KM_BLOCKED 4
#define KM_ECHO 8

#define RAWKEY_L 1
#define RAWKEY_R 2
#define RAWKEY_UP 3
#define RAWKEY_DOWN 4
#define RAWKEY_LEFT 5
#define RAWKEY_RIGHT 6
#define RAWKEY_START 7
#define RAWKEY_SELECT 8
#define RAWKEY_A 9
#define RAWKEY_B 10

#define CHAR_LEFT 241
#define CHAR_RIGHT 242
#define CHAR_LINEKILL 250

#ifdef __cplusplus
}
#endif

#endif
