#ifndef FONT_H
#define FONT_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
	uint16 flags;
	uchar first;
	uchar last;
	uchar charwidth;	// Only for space in proportional fonts
	uchar height;		// Height in pixels
	uint16 width;		// Real width of entire font in pixels - actual planar data is aligned up to nearest 32

	/* Non-disk part */
	char *name;
	uint32 *planar;		// Planar data ((width+31)&0xFFFFFFC0 * height)/8 bytes
	uchar *pixels;		// Chunky data, width*height bytes
	uint16 *offsets;	// NULL for monospaced font
} Font;

/* Disk format

	- Font structure (First part)
	- width*height/8 bytes planar pixels
	- (last-first) 16bit offsets
*/

#define FFLG_PROPORTIONAL 1
#define FFLG_HIFONT 2
#define FFLG_COLOR 4
#define FFLG_TRANSP 8
#define FFLG_BOLD 16
#define FFLG_ITALIC 32

uchar font_putchar(Font *font, char c, int dest, int width);
int font_text(Font *font, char *str, int dest);
int font_text_xy(Font *font, char *str, int x, int y);
void font_setcolor(int fg, int bg);
Font *font_load(char *name);
Font *font_memload(uchar *data);

#ifdef __cplusplus
}
#endif

#endif
