/*
 * Simple Keyboard Device (keyboard.c)
 * Part of libpogo, a c-library replacent for GBA
 * Programmed by Jonas Minnberg (Sasq)
 *
 * DESCRIPTION
 * Stdin device for directly reading the GBA buttons
 *
 **/

#include "core.h"
#include "device.h"
#include "GP32.h"

extern void get_sema(volatile uint32 *sema);
extern void rel_sema(volatile uint32 *sema);
extern int32 try_sema(volatile uint32 *sema);

volatile uint32 reader_sema;

static volatile int lastkeys;
static char inbuf[256];
volatile char *inptr;

int lastcode;
int pressed = -1;
int repcounter = 0;

static Device vkeydev;

static int vkey_open(const char *name, int mode)
{
	return 0;
}

static int vkey_read(int fd, char *dest, int count)
{
	int org = count;
	char *s = inbuf;

	if(inptr == inbuf)
		return 0;

	get_sema(&reader_sema);

	/* Copy from buffer to dest, up to count values */
	while(s < inptr && count) {
		*dest++ = *s++;
		count--;
	}

	/* Copy unread chars to beginning of buffer */
	dest = inbuf;
	while(s < inptr) {
		*dest++ = *s++;
	}
	inptr = dest;

	rel_sema(&reader_sema);

	return org-count;
}

#define KEY_R		(0x8000)
#define KEY_A		(0x4000)
#define KEY_B		(0x2000)
#define KEY_L		(0x1000)
#define KEY_UP		(0x0800)
#define KEY_RIGHT	(0x0400)
#define KEY_DOWN	(0x0200)
#define KEY_LEFT	(0x0100)
#define KEY_SELECT	(0x0080)
#define KEY_START	(0x0040)

#define GET_KEYS() ( ((~rPEDAT) & 0xC0) | ((~rPBDAT) & 0xFF00) )

#define REP_DELAY 10
#define REP_TIME 2

int rep_mask = 0;

/* Call every frame */
static volatile void vkey_update(void)
{
	int diff,i;
	int keys;
	char *before;

	//if(inptr >= &inbuf[256])
	//	return;

	keys = GET_KEYS();

	diff = (keys ^ lastkeys);

	if(!(try_sema(&reader_sema)))
	{
		return;
	}

	if((pressed == keys) && (pressed & rep_mask))
	{
		if(repcounter && (--repcounter == 0))
		{
			*inptr++ = lastcode;
			repcounter = REP_TIME;
		}
	} else
		pressed = -1;

	before = inptr;

	if(diff & KEY_L)
		*inptr++ = (RAWKEY_L | (keys & KEY_L ? 0x00 : 0x80));
	if(diff & KEY_R)
		*inptr++ = (RAWKEY_R | (keys & KEY_R ? 0x00 : 0x80));
	if(diff & KEY_START)
		*inptr++ = (RAWKEY_START | (keys & KEY_START ? 0x00 : 0x80));
	if(diff & KEY_SELECT)
		*inptr++ = (RAWKEY_SELECT | (keys & KEY_SELECT ? 0x00 : 0x80));
	if(diff & KEY_UP)
		*inptr++ = (RAWKEY_UP | (keys & KEY_UP ? 0x00 : 0x80));
	if(diff & KEY_DOWN)
		*inptr++ = (RAWKEY_DOWN | (keys & KEY_DOWN ? 0x00 : 0x80));
	if(diff & KEY_RIGHT)
		*inptr++ = (RAWKEY_RIGHT | (keys & KEY_RIGHT ? 0x00 : 0x80));
	if(diff & KEY_LEFT)
		*inptr++ = (RAWKEY_LEFT | (keys & KEY_LEFT ? 0x00 : 0x80));
	if(diff & KEY_A)
		*inptr++ = (RAWKEY_A | (keys & KEY_A ? 0x00 : 0x80));
	if(diff & KEY_B)
		*inptr++ = (RAWKEY_B | (keys & KEY_B ? 0x00 : 0x80));
	lastkeys = keys;

	if((before != inptr) && (!(*before & 0x80)))
	{
		lastcode = *before;
		pressed = keys;
		repcounter = REP_DELAY;
	}


	rel_sema(&reader_sema);
}

static int vkey_ioctl(int fd, int req, va_list vl)
{
	switch(req) {
	case KC_REPEATMASK:
		rep_mask = va_arg(vl, int);
		break;
	}
	return 0;
}

void key_init(void)
{
	uint16 l;
	memset(&vkeydev, 0, sizeof(vkeydev));
	vkeydev.ioctl = vkey_ioctl;
	vkeydev.open = vkey_open;
	vkeydev.read = (int (*)(int, void *, int))vkey_read;
	device_register(&vkeydev, "/dev/key", vkey_update, 0);

	lastkeys = 0;
	reader_sema = 0;
	repcounter = 0;
	inptr = inbuf;

	rPECON = rPECON & 0xFFFF0FFF;
	rPBCON = rPBCON & 0x0000FFFF;
}
