/*
 * screen.c - screen library
 * Part of libpogo, a c-library replacent for GBA
 * Programmed by Jonas Minnberg (Sasq)
 *
 * DESCRIPTION
 *
 **/


#include "core.h"
#include "device.h"
#include "screen.h"

#include "GP32.h"

static uint16 fg = 0x0000;
static uint16 bg = 0xFFFE;

uint16 *screen = 0x0;
//uint16 *screen = 0xC7B4000;


void mmu_change(void* start_addr, void* end_addr, uint32 flags);

uint16 *screen_offset(int x, int y)
{
	return screen + SCR_BASE + XY(x,y);
}

uint16 *screen_ptr(int pos)
{
	return screen + SCR_BASE + pos;
}

void screen_copy_fg(uint16 *dest, uchar *src, int width, int height, int swidth)
{
	int w;
	int smod = swidth-width;
	int dmod = XY(-width,1);

	while(height--)
	{
		w = width;
		while(w--)
		{
			if(*src++)
				*dest = fg;
			dest += XY(1,0);
		}
		dest += dmod;
		src += smod;
	}
}

void screen_copy(uint16 *dest, uchar *src, int width, int height, int swidth)
{
	int w;
	int smod = swidth-width;
	int dmod = XY(-width,1);

	while(height--)
	{
		w = width;
		while(w--)
		{
			*dest = (*src++ ? fg : bg);
			dest += XY(1,0);
		}
		dest += dmod;
		src += smod;
	}
}

void screen_copy_x(uint16 *dest, uint16 *src, int width, int height, int swidth)
{
	int w;
	int smod = swidth-width;
	int dmod = XY(-width,1);

	while(height--)
	{
		w = width;
		while(w--)
		{
			*dest = *src++;
			dest += XY(1,0);
		}
		dest += dmod;
		src += smod;
	}
}

void screen_clear_bg(uint16 *dest, int width, int height)
{
	int w;
	int dmod = XY(-width,1);

	while(height--)
	{
		w = width;
		while(w--)
		{
			*dest = bg;
			dest += XY(1,0);
		}
		dest += dmod;
	}
}

void screen_set_fgbg(uint16 fgc, uint16 bgc)
{
	fg = fgc;
	bg = bgc;
}

void screen_scroll(int x, int y)
{
	uint16 *dst = screen + SCR_BASE;

	while(1)
	{
		//*dst = *src;
	}

}

void screen_init(void)
{
	int i;
	{
		LCDCON1 lcdcon1;
		memset(&lcdcon1, 0, sizeof(lcdcon1));
		lcdcon1.BPPMODE = 12;		// 16 bits mode
		lcdcon1.CLKVAL = 3;
		lcdcon1.ENVID = 1;
		lcdcon1.MMODE = 0;
		lcdcon1.PNRMODE = 3;
		lcdcon1.LINECNT = 0;
		rLCDCON1 = *(unsigned int *)&lcdcon1;
	}

	{
		LCDCON2 lcdcon2;
		memset(&lcdcon2, 0, sizeof(lcdcon2));
		lcdcon2.LINEVAL = 320-1;
		lcdcon2.VBPD = 1;
		lcdcon2.VFPD = 2;
		lcdcon2.VSPW = 1;
		rLCDCON2 = *(unsigned int *)&lcdcon2;
	}

	{
		LCDCON3 lcdcon3;
		memset(&lcdcon3, 0, sizeof(lcdcon3));
		lcdcon3.HBPD = 6;
		lcdcon3.HFPD = 2;
		lcdcon3.HOZVAL = 240-1;
		rLCDCON3 = *(unsigned int *)&lcdcon3;
	}

	{
		LCDCON4 lcdcon4;
		memset(&lcdcon4, 0, sizeof(lcdcon4));
		lcdcon4.ADDVAL = 0;
		lcdcon4.HSPW = 4;
		lcdcon4.MVAL = 0;
		lcdcon4.PALADDEN = 0;
		rLCDCON4 = *(unsigned int *)&lcdcon4;
	}

	{
		LCDCON5 lcdcon5;

		memset(&lcdcon5, 0, sizeof(lcdcon5));

		lcdcon5.BSWP = 0;
		lcdcon5.ENLEND = 0;
		lcdcon5.HSTATUS = 2;
		lcdcon5.HWSWP = 1;
		lcdcon5.INVENDLINE = 0;
		lcdcon5.INVVCLK = 1;
		lcdcon5.INVVD = 0;
		lcdcon5.INVVDEN = 0;
		lcdcon5.INVVFRAME = 1;
		lcdcon5.INVVLINE = 1;
		lcdcon5.VSTATUS = 2;
		lcdcon5.RESERVED4 = 0;
		rLCDCON5 = *(unsigned int *)&lcdcon5;
	}

	screen = malloc(320*240*2+4096);
	screen = (uint16 *)(((uint32)screen + 4095) & 0xFFFFF000);
	mmu_change(screen, ((uchar*)screen)+320*240*2-1, 0xFF2);

	{
		LCDSADDR1 lcdsaddr1;
		LCDSADDR2 lcdsaddr2;
		LCDSADDR3 lcdsaddr3;

		memset(&lcdsaddr1, 0, sizeof(lcdsaddr1));
		memset(&lcdsaddr2, 0, sizeof(lcdsaddr2));
		memset(&lcdsaddr3, 0, sizeof(lcdsaddr3));

		lcdsaddr1.LCDBANK = (unsigned int)screen >> 22;					// bank
		lcdsaddr1.LCDBASEU = ((unsigned int)screen & 0x3FFFFF) >> 1;	// begin offset (per 16 bits)
		lcdsaddr2.LCDBASEL = lcdsaddr1.LCDBASEU + 320*240;				// end offset (per 16 bits)
		lcdsaddr3.OFFSIZE = 0;
		lcdsaddr3.PAGEWIDTH = 240;
		rLCDSADDR1 = *(unsigned int *)&lcdsaddr1;
		rLCDSADDR2 = *(unsigned int *)&lcdsaddr2;
		rLCDSADDR3 = *(unsigned int *)&lcdsaddr3;
	}

	for(i=0; i<240*320; i++)
		screen[i] = 0xfffe;
}

void screen_set(int bpp, int width, int height)
{
	int realw;
	uint32 scr = (uint32)screen;
	rLCDCON1 = 0x00000301 | bpp;

	width = width >> (4-((bpp&0xF)>>1));
	realw = (320 >> (4-((bpp&0xF)>>1)));

	rLCDSADDR1 = (scr >> 1);
	rLCDSADDR2 = ((scr & 0x3FFFFF) >> 1) + width * height;
	rLCDSADDR3 =  realw | ((width-realw)<<11);
}


void screen_palette(uint16 *colors)
{
	int i;
	uint32 *pal = 0x14a00400;
	for(i=0; i<256; i++)
		pal[i] = colors[i];

}
