#include "core.h"
#include "device.h"

#include "smfs/smf_fat.h"

static Device smfsdev;

//int dir_handles = 0;
//char **dir_names[32];

void make_name(const char *name, char *newname)
{
	char *p;

	sprintf(newname, "dev0:\\%s", name);
	p = newname;
	while(*p)
	{
		if(*p == '/')
			*p = '\\';
		p++;
	}
}

static int ent_start;
char temp_name[256];

static int smfs_open(const char *name, int flags)
{
	ERR_CODE err;
	sFILE_STAT stat;
	F_HANDLE handle;
	char newname[1024];
	unsigned int l;
	int oflags = 0;

	make_name(name, newname);

	if (flags & O_RDWR) oflags |= OPEN_W;
	if (flags & O_APPEND) oflags |= OPEN_W;
	if (flags & O_CREAT)
	{
		if(smCreateFile(newname, (flags & O_TRUNC) ? ALWAYS_CREATE : NOT_IF_EXIST, &handle))
			return -1;
	}
	else
	{
		oflags |= OPEN_R;


		err = smReadStat(newname, &stat);
		if((err == ERR_ROOT_DIR) || ((err == 0) && (stat.attr & 0x10)))
		{
			//printf("opened dir\n");
			ent_start = 0;
			strcpy(temp_name, newname);
			return 0x0F0000FF;
		}
		//printf("stat on %s -> %d, %d\n", newname, err, stat.size);

		if(smOpenFile(newname, oflags, &handle) != 0)
			return -1;
		if(flags & O_APPEND)
			smSeekFile(handle, FROM_END, 0, &l);
	}

	return (int)handle;
}

//SM_EXPORT ERR_CODE smReadDirExHandle(F_HANDLE h_file, udword entry_start, udword entry_count, sDIR_ENTRY_EX p_buf[],	udword* p_read_count);

void unito8(char *d, uint16 *s)
{
	while(*s)
		*d++ = (*s++) & 0xff;
	*d = 0;
}

static int smfs_readdir(int fd, char *dest)
{
	int count;
	sDIR_ENTRY_EX dent;

	//smReadDirExHandle((F_HANDLE)fd, ent_start, 1, &dent, &count);
	smReadDirEx(temp_name, ent_start, 1, &dent, &count);


	if(count)
	{
		if(strlen(dent.long_name))
		{
			//printf("Long :%c %c %c %c:\n", dent.long_name[0], dent.long_name[1], dent.long_name[2], dent.long_name[3]);
			unito8(dest, dent.long_name);
		}
		else
		{
			char *d = dest;
			char *s = dent.name;

			*d++ = *s++;
			while(*s)
				*d++ = tolower(*s++);
			*d++ = 0;
			//strcpy(dest, dent.name);
		}

		//printf("Read %d from %s -> %s\n", count, temp_name, dest);
		ent_start++;

		return strlen(dest);
	}

	return -1;
}


static int smfs_read(int fd, void *dest, int size)
{
	unsigned int read_count;
	smReadFile((F_HANDLE)fd, dest, size, &read_count);
	return read_count;
}

static int smfs_write(int fd, const void *src, int size)
{
	// align
	//void *src2=malloc(size);
	//memcpy(src2, src, size);
	// when writing more then 16384 bytes at a time
	// smWriteFile seems to output 2 bytes of garbage at 16385
	/*int block_size=16384;
	int bytes_left,bytes_written;
	unsigned int sm_offset = -1;
	int block_num=0;
	int compensate;

	bytes_left=size;
	bytes_written=0;
	while(bytes_left>0) {
		compensate=block_num*2;
		if(bytes_left>block_size) {
			//if(smWriteFile((F_HANDLE)fd, 
			//		src+bytes_written, block_size) == 0)
			//	return -1;
			smWriteFile((F_HANDLE)fd, 
					src+bytes_written-compensate, 
					block_size);
			bytes_left-=block_size;
			bytes_written+=block_size;
			// write 2 bytes and rewind the pointer 2
			//smWriteFile((F_HANDLE)fd, '\0', 1);
			//smWriteFile((F_HANDLE)fd, '\0', 1);
			//smSeekFile((F_HANDLE)fd, FROM_CURRENT, -2, &sm_offset);
		} else {
			//if(smWriteFile((F_HANDLE)fd, 
			//		src+bytes_written, bytes_left) == 0)
			//	return -1;
			smWriteFile((F_HANDLE)fd, 
					src+bytes_written-compensate, 
					bytes_left);
			bytes_left=0;
		}
		block_num++;
	}
	return size;
	*/

	if(smWriteFile((F_HANDLE)fd, src, size) == 0)
		return -1;
	return size;
}

static int smfs_close(int fd)
{
	return smCloseFile((F_HANDLE)fd);
}

static int smfs_stat(const char *name, struct stat *st)
{
	ERR_CODE err;
	sFILE_STAT stat;
	//struct tm tim;
	char newname[1024];

	make_name(name, newname);

	//strcpy(newname, "dev0:");

	//printf("stat on \"%s\"\n", newname);

	if((err = smReadStat(newname, &stat)) != 0)
	{
		if(err == ERR_ROOT_DIR)
		{
			//printf("rootdir\n");

			memset(st, 0, sizeof(struct stat));
			st->st_mode = S_IREAD | S_IFDIR;
			st->st_nlink = 1;
			return 0;

		}
		return -1;
	}
/*
 *	7-6 Reserved. Must be 0's.
 *	5: 1 = ARCHIVE file was modified
 *	4: 1 = DIRECTORY, 0 = file
 *	3: 1 = VOLUME label
 *	2: 1 = SYSTEM file or directory
 *	1: 1 = HIDDEN file or directory
 *	0: 1 = READONLY file
 */
	memset(st, 0, sizeof(struct stat));
	st->st_mode =
		S_IEXEC +
		S_IREAD +
		((stat.attr & 0x01) ? 0 : S_IWRITE) +			// readonly
		((stat.attr & 0x02) ? 0 : 0) +					// hidden
		((stat.attr & 0x04) ? 0 : 0) +					// system
		((stat.attr & 0x08) ? 0 : 0) +					// volume label
		((stat.attr & 0x10) ? S_IFDIR : 0) +			// directory
		((stat.attr & 0x20) ? 0 : 0);					// archive
	st->st_size = stat.size;
	st->st_ino = stat.cluster;
	st->st_nlink = 1;
/*	tim.tm_sec = stat.time.sec;
	tim.tm_min = stat.time.min;
	tim.tm_hour = stat.time.hour;
	tim.tm_mday = stat.time.day;
	tim.tm_mon = stat.time.month;
	tim.tm_year = stat.time.year;
	tim.tm_wday = 0;
	tim.tm_yday = 0;
	tim.tm_isdst = 0;
	st->st_atime = mktime(&tim);*/
	st->st_atime = 0;
	//st->st_blksize = S_BLKSIZE;

	//printf("mode is %x\n", st->st_mode);

	return 0;
}


static int smfs_lseek(int fd, int offset, int origin)
{
	int mode = -1;
	unsigned int sm_offset = -1;

	switch(origin)
	{
	case SEEK_SET : mode = FROM_BEGIN;
		break;
	case SEEK_CUR : mode = FROM_CURRENT;
		break;
	case SEEK_END : mode = FROM_END;
		break;
	}

	if(smSeekFile((F_HANDLE)fd, mode, offset, &sm_offset) != 0)
		return -1;
	smSeekFile((F_HANDLE)fd, FROM_CURRENT, 0, &sm_offset);
	return sm_offset;
}

static int smfs_remove(const char *name)
{
	sFILE_STAT stat;
	char newname[1024];
	make_name(name, newname);
}

static int smfs_rename(const char *oldname, const char *newname)
{

}

void filesys_init(void)
{
	int rc = smInit();

	if(rc)
	{
//		printf("init = %d\n");
	}

	memset(&smfsdev, 0, sizeof(smfsdev));
	smfsdev.open = smfs_open;
	smfsdev.close = smfs_close;
	smfsdev.read = smfs_read;
	smfsdev.write = smfs_write;
	smfsdev.lseek = smfs_lseek;
	smfsdev.stat = smfs_stat;
	smfsdev.remove = smfs_remove;
	smfsdev.rename = smfs_rename;
	smfsdev.dread = smfs_readdir;

	device_register(&smfsdev, "/smfs/", NULL, -1);
}
