

extern "C" {
#include "input.h"
#include "pogo.h"
#include "ui.h"
#include "gpmisc.h"
#include "menu.h"
}

#include "C64.h"
#include "Prefs.h"
//#include "SAM.h"

extern Prefs ThePrefs;
extern int current_joystick;

extern uint8 charrom[];
uint8 *font=charrom;

#define MATRIX(a,b) (((a) << 3) | (b))
int vkey_pressed=0;
int vkey_released=0;
int vkey=0;

int cursor_x=15;
int cursor_x_last=15;
int cursor_y=20;
int cursor_y_last=20;

struct dir_item {
	char *name;
	int type;
};

int ascii2charrom(char chr) {
	int x=(int)chr;

	if(x>=97&&x<=122) x+=160;
	else if(x>=65&&x<=90) x-=64;
	return x;
}

void text_out8(uint8 *screen, const char *string, uint8 bg, uint8 fg, 
		int x, int y, int transparant) {
	int char_width=DISPLAY_WIDTH/FONT_WIDTH;
	int xpos;
	int xx;
	int yy;
	uint8 pixel;
	unsigned char *character_data;
	int i;
	int screen_x, screen_y;

	if(y>=(DISPLAY_HEIGHT/FONT_HEIGHT)) return;
	xpos=x;
	for(i=0; (xpos<char_width)&&string[i]; i++) {
		character_data=font+(ascii2charrom(string[i])
				*((FONT_WIDTH/8)*FONT_HEIGHT));
		xx=0;
		yy=0;
		while(yy<FONT_HEIGHT) {
			pixel=*(character_data+yy);
			pixel=pixel>>((FONT_WIDTH-1)-xx);
			if(pixel&1) {
				pixel=fg;
			} else {
				pixel=bg;
			}
			if(!(transparant&&(pixel==bg)&&(xx&1)&&(yy&1))) {
				screen_x=(BORDER_LEFT+(xpos*FONT_WIDTH))+xx;
				screen_y=(BORDER_TOP+(y*FONT_HEIGHT))+yy;
				*(screen+(BUFFER_HEIGHT*screen_x)
						+((BUFFER_HEIGHT-1)-screen_y))
						=pixel;
			}

			xx++;
			if(xx==FONT_WIDTH) {
				xx=0;
				yy++;
			}
		}
		xpos++;
	}
}

void sort_dir(struct dir_item *list, int num_items, int sepdir) {
	int i;
	struct dir_item temp;

	for(i=0; i<(num_items-1); i++) {
		if(strcmp(list[i].name, list[i+1].name)>0) {
			temp=list[i];
			list[i]=list[i+1];
			list[i+1]=temp;
			i=0;
		}
	}
	if(sepdir) {
		for(i=0; i<(num_items-1); i++) {
			if((list[i].type==1)&&(list[i+1].type==0)) {
				temp=list[i];
				list[i]=list[i+1];
				list[i+1]=temp;
				i=0;
			}
		}
	}
}

char *filerequest(char *dir) {
	static char *cwd=NULL;
	static int num_items=0;
	struct dir_item dir_items[1024];
	struct stat item;
	static int fcursor_pos;
	static int first_visible;
	static int row, column;
	DIR *dirstream;
	struct dirent *direntry;
	char *path;
	uint16 bg,fg;
	int i;
	int pathlength;
	char *blank_line="                      ";
	int menu_x=5;
	int menu_y=3;
	int menu_height=23;
	static int is_moving=0;
	static int threshold=10;
	static int stepping=2;

	uint8 menu_bg=14;
	uint8 menu_fg=6;
	uint8 menu_selbg=7;
	uint8 menu_selfg=6;

	if(dir!=NULL) cwd=dir;
	if(cwd==NULL) cwd="/smfs/gpmm/c64";

	if(num_items==0) {
		dirstream=opendir(cwd);
		if(dirstream==NULL) {
			button_state&=~A_PRESSED;
			printf("error opening directory %s\n", cwd);
			return (char *)-1;
		}
		// read directory entries
		while(direntry=readdir(dirstream)) {
			dir_items[num_items].name=(char *)malloc(
					strlen(direntry->d_name)+1);
			strcpy(dir_items[num_items].name, direntry->d_name);
			num_items++;
			if(num_items>1024) break;
		}
		closedir(dirstream);
		// get entry types
		for(i=0; i<num_items; i++) {
			path=(char *)malloc(strlen(cwd)
					+strlen(dir_items[i].name)
					+2);
			sprintf(path, "%s/%s", cwd, dir_items[i].name);
			if(!stat(path, &item)) {
				if(S_ISDIR(item.st_mode)) {
					dir_items[i].type=0;
				} else {
					dir_items[i].type=1;
				}
			} else {
				dir_items[i].type=0;
			}
			free(path);
		}
		sort_dir(dir_items, num_items, 1);
		fcursor_pos=0;
		first_visible=0;
	}

	// display current directory
	text_out8(emu_screen, cwd+6, menu_bg, menu_fg, 5, menu_y-1, 0);

	// display directory contents
	row=0;
	while(row<num_items
			//&& row<(DISPLAY_HEIGHT/FONT_HEIGHT)) {
			&& row<menu_height) {
		column=menu_x;
		if(row==(fcursor_pos-first_visible)) {
			bg=menu_selbg;
			fg=menu_selfg;
		} else {
			bg=menu_bg;
			fg=menu_fg;
		}
		if(dir_items[row+first_visible].type==0) {
			text_out8(emu_screen, 
					"<DIR> ", 
				bg, fg, column, menu_y+row, 0);
		} else {
			text_out8(emu_screen, 
					"      ", 
				bg, fg, column, menu_y+row, 0);
		}
		column+=6;
		text_out8(emu_screen, blank_line, 
				bg, fg, column, menu_y+row, 0);
		text_out8(emu_screen, dir_items[row+first_visible].name, 
				bg, fg, column, menu_y+row, 0);
		row++;
	}
	if(button_state&DOWN_PRESSED) {
		if(is_moving==0||((is_moving>threshold)&&(is_moving%stepping==0))) {
			if(fcursor_pos<(num_items-1)) fcursor_pos++;
			if((fcursor_pos-first_visible) >=menu_height) first_visible++;
		}
		is_moving++;
	} else if(button_state&UP_PRESSED) {
		if(is_moving==0||((is_moving>threshold)&&(is_moving%stepping==0))) {
			if(fcursor_pos>0) fcursor_pos--;
			if(fcursor_pos<first_visible) first_visible--;
		}
		is_moving++;
	} else {
		is_moving=0;
	}
	
	if(button_state&A_PRESSED) {
		button_state&=~A_PRESSED;
		path=(char *)malloc(strlen(cwd)
				+strlen(dir_items[fcursor_pos].name)
				+2);
		sprintf(path, "%s/%s", cwd, dir_items[fcursor_pos].name);
		for(i=0; i<num_items; i++) free(dir_items[i].name);
		num_items=0;
		if(dir_items[fcursor_pos].type==0) {
			// directory selected
			pathlength=strlen(path);
			if(path[pathlength-1]=='.'
				// check for . selected
					&& path[pathlength-2]=='/') {
				path[pathlength-2]='\0';
				return path;
			} else if(path[pathlength-1]=='.'
				// check for .. selected
					&& path[pathlength-2]=='.'
					&& path[pathlength-3]=='/'
					&&pathlength>8) {
				for(i=4; path[pathlength-i]!='/'; i++);
				path[pathlength-i]='\0';
				if(strlen(path)<=5) {
					path[5]='/';
					path[6]='\0';
				}
				cwd=path;
			} else {
				cwd=path;
			}
		} else {
			// file selected
			return path;
		}
	} else if(button_state&B_PRESSED) {
		return (char *)-1;
	}
	return NULL;
}

void draw_options(C64 *TheC64) {
	char *imagefile;
	int i;
	static int cursor_pos=3;
	static Prefs *prefs=NULL;
	int update_prefs=0;
	uint16 bg, fg;
	static int getfilename;
	static int xfiletype;
	static int menu_y=3;
	static int menu_x=5;
	static int is_moving=0;
	static int threshold=10;
	static int stepping=2;

	enum { 
		D64, T64, DIRFS, LOADSTAR, LOADER,
		BLANK1,
		SOUND, BRIGHTNESS, CONTRAST, FRAMESKIP, LIMITSPEED,
		JOYSTICK, EMU1541CPU, 
		BLANK2,
		LOAD_SNAP, SAVE_SNAP,
		BLANK3,
		RESET64, DEBUG, RESETGP,
		NUM_OPTIONS 
	};

	char *option_txt[NUM_OPTIONS+5];
	option_txt[SOUND]=		"Sound                       ";
	option_txt[BRIGHTNESS]= 	"Brightness                  ";
	option_txt[CONTRAST]= 		"Contrast                    ";
	option_txt[FRAMESKIP]=		"Frameskip                   ";
	option_txt[LIMITSPEED]=		"Limit speed                 ";
	option_txt[JOYSTICK]=		"Joystick in port            ";
	option_txt[EMU1541CPU]=		"1541 CPU emulation          ";
	option_txt[D64]=		"Attach d64 image...         ";
	option_txt[T64]=		"Attach t64 image...         ";
	option_txt[DIRFS]=		"Attach directory...         ";
	option_txt[LOADSTAR]=		"Load first program          ";
	option_txt[LOADER]=		"Load d64/dir browser        ";
	option_txt[SAVE_SNAP]=		"Save snapshot               ";
	option_txt[LOAD_SNAP]=		"Load snapshot               ";
	option_txt[RESET64]=		"Reset C=64                  ";
	option_txt[DEBUG]=		"Debug console               ";
	option_txt[RESETGP]=		"Restart GP32                ";
	option_txt[BLANK1]=		"                            ";
	option_txt[BLANK2]=		"                            ";
	option_txt[BLANK3]=		"                            ";
	option_txt[NUM_OPTIONS]=	"                            ";
	option_txt[NUM_OPTIONS+1]=	"                            ";
	option_txt[NUM_OPTIONS+2]=	"                            ";
	option_txt[NUM_OPTIONS+3]=	"                            ";
	option_txt[NUM_OPTIONS+4]=	NULL;

	char *brightness_txt="  ";
	char *contrast_txt=" . ";
	char *frameskip_txt="  ";

	uint8 menu_bg=14;
	uint8 menu_fg=6;
	uint8 menu_selbg=7;
	uint8 menu_selfg=6;

	if(getfilename) {
		imagefile=filerequest(NULL);
		if(imagefile==NULL) {
			return;
		} else if (imagefile==(char *)-1) {
			getfilename=0;
		} else {
			if(xfiletype==0) prefs->DriveType[0]=DRVTYPE_D64;
			if(xfiletype==1) prefs->DriveType[0]=DRVTYPE_T64;
			if(xfiletype==2) prefs->DriveType[0]=DRVTYPE_DIR;
			strcpy(prefs->DrivePath[0], imagefile);
			update_prefs=1;
			getfilename=0;
			cursor_pos=menu_y+LOADSTAR;
		}
	}

	if(prefs==NULL) prefs=new Prefs(ThePrefs);

	for(i=0; option_txt[i]; i++) {
		if(cursor_pos==(menu_y+i)) { 
			bg=menu_selbg;
			fg=menu_selfg; 
		} else { 
			bg=menu_bg;
			fg=menu_fg; 
		}
		text_out8(emu_screen, option_txt[i], bg, fg, menu_x, menu_y+i, 0);
	}

	// sound state
	if(prefs->SIDType==SIDTYPE_NONE) { 
		bg=menu_selbg;
		fg=menu_selfg; 
	} else { 
		bg=menu_bg;
		fg=menu_fg; 
	}
	text_out8(emu_screen, "Off ",
			bg, fg, menu_x+20, menu_y+SOUND, 0);
	if(prefs->SIDType==SIDTYPE_DIGITAL) { 
		bg=menu_selbg;
		fg=menu_selfg; 
	} else { 
		bg=menu_bg;
		fg=menu_fg; 
	}
	text_out8(emu_screen, "On  ",
			bg, fg, menu_x+24, menu_y+SOUND, 0);
	bg=menu_bg;
	fg=menu_fg;

	// brightness
	brightness_txt[0]=(char)(0x30+((brightness%100)/10));
	brightness_txt[1]=(char)(0x30+((brightness%100)%10));
	text_out8(emu_screen, brightness_txt,
			bg, fg, menu_x+20, menu_y+BRIGHTNESS, 0);
	// contrast
	contrast_txt[0]=(char)(0x30+((contrast%100)/10));
	contrast_txt[2]=(char)(0x30+((contrast%100)%10));
	text_out8(emu_screen, contrast_txt,
			bg, fg, menu_x+20, menu_y+CONTRAST, 0);

	// frameskip
	frameskip_txt[0]=(char)(0x30+(((prefs->SkipFrames-1)%100)/10));
	frameskip_txt[1]=(char)(0x30+(((prefs->SkipFrames-1)%100)%10));
	text_out8(emu_screen, frameskip_txt,
			bg, fg, menu_x+20, menu_y+FRAMESKIP, 0);

	// limit speed
	if(prefs->LimitSpeed==false) { 
		bg=menu_selbg;
		fg=menu_selfg; 
	} else { 
		bg=menu_bg;
		fg=menu_fg; 
	}
	text_out8(emu_screen, "Off ",
			bg, fg, menu_x+20, menu_y+LIMITSPEED, 0);
	if(prefs->LimitSpeed==true) { 
		bg=menu_selbg;
		fg=menu_selfg; 
	} else { 
		bg=menu_bg;
		fg=menu_fg; 
	}
	text_out8(emu_screen, "On  ",
			bg, fg, menu_x+24, menu_y+LIMITSPEED, 0);
	bg=menu_bg;
	fg=menu_fg;

	// current joyport
	if(current_joystick==0) { 
		bg=menu_selbg;
		fg=menu_selfg; 
	} else { 
		bg=menu_bg;
		fg=menu_fg; }
	text_out8(emu_screen, "1   ",
			bg, fg, menu_x+20, menu_y+JOYSTICK, 0);
	if(current_joystick==1) { 
		bg=menu_selbg;
		fg=menu_selfg; 
	} else { 
		bg=menu_bg;
		fg=menu_fg; 
	}
	text_out8(emu_screen, "2   ",
			bg, fg, menu_x+24, menu_y+JOYSTICK, 0);

	// 1541 cpu
	if(prefs->Emul1541Proc==false) { 
		bg=menu_selbg;
		fg=menu_selfg; 
	} else { 
		bg=menu_bg;
		fg=menu_fg; }
	text_out8(emu_screen, "Off ",
			bg, fg, menu_x+20, menu_y+EMU1541CPU, 0);
	if(prefs->Emul1541Proc==true) { 
		bg=menu_selbg;
		fg=menu_selfg; 
	} else { 
		bg=menu_bg;
		fg=menu_fg; 
	}
	text_out8(emu_screen, "On  ",
			bg, fg, menu_x+24, menu_y+EMU1541CPU, 0);

	// reset type
	if(prefs->FastReset==false) { 
		bg=menu_selbg;
		fg=menu_selfg; 
	} else { 
		bg=menu_bg;
		fg=menu_fg; }
	text_out8(emu_screen, "slow",
			bg, fg, menu_x+20, menu_y+RESET64, 0);
	if(prefs->FastReset==true) { 
		bg=menu_selbg;
		fg=menu_selfg; 
	} else { 
		bg=menu_bg;
		fg=menu_fg; 
	}
	text_out8(emu_screen, "fast",
			bg, fg, menu_x+24, menu_y+RESET64, 0);

	i=menu_y+NUM_OPTIONS+1;
	// display attached file
	bg=menu_bg;
	fg=menu_fg;
	if(prefs->DriveType[0]==DRVTYPE_D64) {
		text_out8(emu_screen, "drive 8 D64: ", bg, fg, menu_x, i++, 0);
		text_out8(emu_screen, prefs->DrivePath[0]+5, bg, fg, menu_x, i++, 0);
	} else if(prefs->DriveType[0]==DRVTYPE_T64) {
		text_out8(emu_screen, "drive 8 T64: ", bg, fg, menu_x, i++, 0);
		text_out8(emu_screen, prefs->DrivePath[0]+5, bg, fg, menu_x, i++, 0);
	} else if(prefs->DriveType[0]==DRVTYPE_DIR) {
		text_out8(emu_screen, "drive 8 directory: ", bg, fg, menu_x, i++, 0);
		text_out8(emu_screen, prefs->DrivePath[0]+5, bg, fg, menu_x, i++, 0);
	}
	text_out8(emu_screen, "Press select to load", bg, fg, menu_x, i++, 0);

	if(button_state&A_PRESSED) {
		button_state&=~(A_PRESSED);
		if(cursor_pos==menu_y+LOADSTAR) {
			kbd_buf_feed("\rLOAD\":*\",8,1:\rRUN\r");
			options_enabled=0;
		}
		if(cursor_pos==menu_y+LOADER) {
			kbd_buf_feed("\rLOAD\"LOADER\",11,1\rRUN\r");
			options_enabled=0;
		}
		if(cursor_pos==menu_y+D64) {
			getfilename=1;
			xfiletype=0;
		} else if(cursor_pos==menu_y+T64) {
			getfilename=1;
			xfiletype=1;
		} else if(cursor_pos==menu_y+DIRFS) {
			getfilename=1;
			xfiletype=2;
		} else if(cursor_pos==menu_y+RESET64) {
			TheC64->Reset();
		} else if(cursor_pos==menu_y+SAVE_SNAP) {
			TheC64->Pause();
			TheC64->SaveSnapshot("/smfs/gpmm/c64/frodo/test.sna");
			TheC64->Resume();
		} else if(cursor_pos==menu_y+LOAD_SNAP) {
			TheC64->Pause();
			TheC64->LoadSnapshot("/smfs/gpmm/c64/frodo/test.sna");
			TheC64->Resume();
		} else if(cursor_pos==menu_y+DEBUG) {
			switch_console();
		/*} else if(cursor_pos==10) {
			app_execute(prefs->DrivePath[0]);*/
		} else if(cursor_pos==menu_y+RESETGP) {
			gp32reboot();
		}
	} 
	
	if(button_state&UP_PRESSED) {
		if(is_moving==0||((is_moving>threshold)&&(is_moving%stepping==0))) {
			if(cursor_pos>menu_y) cursor_pos--;
		}
		is_moving++;
	} else if(button_state&DOWN_PRESSED) {
		if(is_moving==0||((is_moving>threshold)&&(is_moving%stepping==0))) {
			if(cursor_pos<menu_y+NUM_OPTIONS-1) cursor_pos++;
		}
		is_moving++;
	} else {
		is_moving=0;
	}

	if(button_state&LEFT_PRESSED) {
		if(cursor_pos==menu_y+SOUND) {
			prefs->SIDType=SIDTYPE_NONE; 
			update_prefs=1;
		}
		if(cursor_pos==menu_y+BRIGHTNESS) {
			if(brightness>0) {
				brightness--;
				ui_set_palette();
			}
		}
		if(cursor_pos==menu_y+CONTRAST) {
			if(contrast>0) {
				contrast--;
				ui_set_palette();
			}
		}
		if(cursor_pos==menu_y+FRAMESKIP) {
			if(prefs->SkipFrames>1) {
				prefs->SkipFrames--;
				update_prefs=1;
			}
		}
		if(cursor_pos==menu_y+LIMITSPEED) {
			prefs->LimitSpeed=false;
			update_prefs=1;
		}
		if(cursor_pos==menu_y+JOYSTICK) {
			current_joystick=0;
		}
		if(cursor_pos==menu_y+EMU1541CPU) {
			prefs->Emul1541Proc=false;
			update_prefs=1;
		}
		if(cursor_pos==menu_y+RESET64) {
			prefs->FastReset=false;
			update_prefs=1;
		}
	} else if(button_state&RIGHT_PRESSED) {
		if(cursor_pos==menu_y+SOUND) {
			prefs->SIDType=SIDTYPE_DIGITAL; 
			update_prefs=1;
		}
		if(cursor_pos==menu_y+BRIGHTNESS) {
			if(brightness<25) {
				brightness++;
				ui_set_palette();
			}
		}
		if(cursor_pos==menu_y+CONTRAST) {
			if(contrast<100) {
				contrast++;
				ui_set_palette();
			}
		}
		if(cursor_pos==menu_y+FRAMESKIP) {
			if(prefs->SkipFrames<100) {
				prefs->SkipFrames++;
				update_prefs=1;
			}
		}
		if(cursor_pos==menu_y+LIMITSPEED) {
			prefs->LimitSpeed=true;
			update_prefs=1;
		}
		if(cursor_pos==menu_y+JOYSTICK) {
			current_joystick=1;
		}
		if(cursor_pos==menu_y+EMU1541CPU) {
			prefs->Emul1541Proc=true;
			update_prefs=1;
		}
		if(cursor_pos==menu_y+RESET64) {
			prefs->FastReset=true;
			update_prefs=1;
		}
	}

	if(update_prefs) {
		TheC64->NewPrefs(prefs);
		ThePrefs=*prefs;
		delete prefs;
		prefs=NULL;
		update_prefs=0;
	}
}

void draw_keyboard(C64 *TheC64) {
	const int keyb_width=27;
	const int keyb_height=8;
	static int keyb_start_x=14;
	static int keyb_start_y=21;
	static int shifted=0;
	static int rstopd=0;
	static int ctrled=0;
	static int rstred=0;
	static int gotkeypress;
	static int keyb_grabbed=0;
	int i;

	char keyb[keyb_height][keyb_width]={
		" =---=================*= ",
		"                         ",
		"    < 1234567890 del  F1 ",
		" ctrl QWERTYUIOP rstr F3 ",
		" r/st ASDFGHJKL; rtrn F5 ",
		" shft ZXCVBNM,./   .  F7 ",
		"  c    space      ...    ",
		"                         "
	};

	int keytable[] = {
		//x, y, key value
		6, 2, MATRIX(7,0), //1
		7, 2, MATRIX(7,3), //2
		8, 2, MATRIX(1,0), //3
		9, 2, MATRIX(1,3), //4
		10, 2, MATRIX(2,0), //5
		11, 2, MATRIX(2,3), //6
		12, 2, MATRIX(3,0), //7
		13, 2, MATRIX(3,3), //8
		14, 2, MATRIX(4,0), //9
		15, 2, MATRIX(4,3), //0

		6, 3, MATRIX(7,6), //q
		7, 3, MATRIX(1,1), //w
		8, 3, MATRIX(1,6), //e
		9, 3, MATRIX(2,1), //r
		10, 3, MATRIX(2,6), //t
		11, 3, MATRIX(3,1), //y
		12, 3, MATRIX(3,6), //u
		13, 3, MATRIX(4,1), //i
		14, 3, MATRIX(4,6), //o
		15, 3, MATRIX(5,1), //p

		6, 4, MATRIX(1,2), //a
		7, 4, MATRIX(1,5), //s
		8, 4, MATRIX(2,2), //d
		9, 4, MATRIX(2,5), //f
		10, 4, MATRIX(3,2), //g
		11, 4, MATRIX(3,5), //h
		12, 4, MATRIX(4,2), //j
		13, 4, MATRIX(4,5), //k
		14, 4, MATRIX(5,2), //l
		15, 4, MATRIX(6,2), //;

		6, 5, MATRIX(1,4), //z
		7, 5, MATRIX(2,7), //x
		8, 5, MATRIX(2,4), //c
		9, 5, MATRIX(3,7), //v
		10, 5, MATRIX(3,4), //b
		11, 5, MATRIX(4,7), //n
		12, 5, MATRIX(4,4), //m
		13, 5, MATRIX(5,7), //,
		14, 5, MATRIX(5,4), //.
		15, 5, MATRIX(6,7), ///

		2, 6, MATRIX(7,5), //c=

		//1, 4, MATRIX(7,7), //run/stop
		//2, 4, MATRIX(7,7), //
		//3, 4, MATRIX(7,7), //
		//4, 4, MATRIX(7,7), //

		4, 2, MATRIX(7,1), //<-

		7,6, MATRIX(7,4), // space
		8,6, MATRIX(7,4), // space
		9,6, MATRIX(7,4), // space
		10,6, MATRIX(7,4), // space
		11,6, MATRIX(7,4), // space

		17, 4, MATRIX(0,1), //return
		18, 4, MATRIX(0,1), //return
		19, 4, MATRIX(0,1), //return
		20, 4, MATRIX(0,1), //return

		19, 5, (MATRIX(0,7))|128, //crsr up
		19, 6, MATRIX(0,7), //crsr down
		18, 6, (MATRIX(0,2))|128, //crsr left
		20, 6, MATRIX(0,2), //crsr right

		17, 2, MATRIX(0,0), //delete
		18, 2, MATRIX(0,0), //delete
		19, 2, MATRIX(0,0), //delete

		22, 2, MATRIX(0,4), //f1
		23, 2, MATRIX(0,4), //f1
		22, 3, MATRIX(0,5), //f3
		23, 3, MATRIX(0,5), //f3
		22, 4, MATRIX(0,6), //f5
		23, 4, MATRIX(0,6), //f5
		22, 5, MATRIX(0,3), //f7
		23, 5, MATRIX(0,3), //f7

		NULL
	};


	text_out8(emu_screen, keyb[0], 9, 1, keyb_start_x, keyb_start_y, 1);
	text_out8(emu_screen, keyb[1], 9, 1, keyb_start_x, keyb_start_y+1, 1);
	text_out8(emu_screen, keyb[2], 9, 1, keyb_start_x, keyb_start_y+2, 1);
	text_out8(emu_screen, keyb[3], 9, 1, keyb_start_x, keyb_start_y+3, 1);
	text_out8(emu_screen, keyb[4], 9, 1, keyb_start_x, keyb_start_y+4, 1);
	text_out8(emu_screen, keyb[5], 9, 1, keyb_start_x, keyb_start_y+5, 1);
	text_out8(emu_screen, keyb[6], 9, 1, keyb_start_x, keyb_start_y+6, 1);
	text_out8(emu_screen, keyb[7], 9, 1, keyb_start_x, keyb_start_y+7, 1);

	// shift key
	if(shifted) {
		text_out8(emu_screen, "shft", 7, 1, keyb_start_x+1, keyb_start_y+5, 0);
	}
	// run/stop key
	if(rstopd) {
		text_out8(emu_screen, "r/st", 7, 1, keyb_start_x+1, keyb_start_y+4, 0);
	}
	// ctrl key
	if(ctrled) {
		text_out8(emu_screen, "ctrl", 7, 1, keyb_start_x+1, keyb_start_y+3, 0);
	}
	// power led :)
	text_out8(emu_screen, "*", 9, 2, keyb_start_x+22, keyb_start_y, 1);

	if(button_state&A_PRESSED) {
		// drag keyboard
		if(cursor_x>=keyb_start_x
				&&cursor_x<=(keyb_start_x+keyb_width)
				&&cursor_y==keyb_start_y
				&&cursor_x==cursor_x_last
				&&cursor_y==cursor_y_last
				) {
			keyb_grabbed=1;
		}
		/*
			if((cursor_x<cursor_x_last)
					&&(cursor_y==keyb_start_y)
					&& keyb_start_x>0) {
				keyb_start_x--;
			} else if((cursor_x>cursor_x_last)
					&&(cursor_y==keyb_start_y)
					&& keyb_start_x<30) {
				keyb_start_x++;
			}

			if((cursor_y<cursor_y_last)
					&&(cursor_y==keyb_start_y)
					&& keyb_start_y>0) {
				keyb_start_y--;
			} else if((cursor_y>cursor_y_last)
					&&(cursor_y==keyb_start_y)
					&& keyb_start_y<24) {
				keyb_start_y++;
			}*/
			/*if((button_state&LEFT_PRESSED)
					&&(cursor_y==keyb_start_y)
					&& keyb_start_x>0) {
				keyb_start_x--;
			} else if((button_state&RIGHT_PRESSED)
					&&(cursor_y==keyb_start_y)
					&& keyb_start_x<30) {
				keyb_start_x++;
			}

			if((button_state&UP_PRESSED)
					&&(cursor_y==keyb_start_y)
					&& keyb_start_y>0) {
				keyb_start_y--;
			} else if((button_state&DOWN_PRESSED)
					&&(cursor_y==keyb_start_y)
					&& keyb_start_y<24) {
				keyb_start_y++;
			}*/
		//}

		// key input
		for(i=0; keytable[i]; i+=3) {
			if((cursor_x==keyb_start_x+keytable[i])
					&& (cursor_y==keyb_start_y+keytable[i+1])) {
				vkey=keytable[i+2];
				if(shifted) vkey|=128;
				vkey_pressed=1;
				gotkeypress=1;
				break;
			}
		}
		// shift key
		if(cursor_y==keyb_start_y+5
				&&cursor_x>=keyb_start_x+1
				&&cursor_x<=keyb_start_x+4) {
			shifted^=1;
			button_state&=~(A_PRESSED);
		}
		// ctrl key
		  else if(cursor_y==keyb_start_y+3
				&&cursor_x>=keyb_start_x+1
				&&cursor_x<=keyb_start_x+4) {
			if(ctrled==0) {
				ctrled=1;
				vkey=MATRIX(7,2);
				vkey_pressed=1;
				gotkeypress=1;
			} else {
				ctrled=0;
				vkey=MATRIX(7,2);
				vkey_pressed=0;
				gotkeypress=0;
				vkey_released=1;
			}

			button_state&=~(A_PRESSED);
		}
		// restore key
		  else if(cursor_y==keyb_start_y+3
				&&cursor_x>=keyb_start_x+17
				&&cursor_x<=keyb_start_x+20) {
			TheC64->NMI();
			button_state&=~(A_PRESSED);
		}
		// run/stop key
		  else if(cursor_y==keyb_start_y+4
				&&cursor_x>=keyb_start_x+1
				&&cursor_x<=keyb_start_x+4) {
			if(rstopd==0) {
				rstopd=1;
				vkey=MATRIX(7,7);
				if(shifted) vkey|=128;
				vkey_pressed=1;
				gotkeypress=1;
			} else {
				rstopd=0;
				vkey=MATRIX(7,7);
				if(shifted) vkey|=128;
				vkey_pressed=0;
				gotkeypress=0;
				vkey_released=1;
			}

			button_state&=~(A_PRESSED);
		}

	} else {
		keyb_grabbed=0;
		if(gotkeypress
				&& vkey!=MATRIX(7,7)
				&& vkey!=MATRIX(7,2)) {
			gotkeypress=0;
			vkey_pressed=0;
			vkey_released=1;
		}
	}

	if(keyb_grabbed) {
		if((cursor_x<cursor_x_last)
				&& keyb_start_x>0) {
			keyb_start_x--;
		} else if((cursor_x>cursor_x_last)
				&& keyb_start_x<30) {
			keyb_start_x++;
		}

		if((cursor_y<cursor_y_last)
				&& keyb_start_y>0) {
			keyb_start_y--;
		} else if((cursor_y>cursor_y_last)
				&& keyb_start_y<24) {
			keyb_start_y++;
		}
	}
}

void draw_status() {
	static char *time="00:00";
	static char *speed="000%";
	static char *fps="00";
	uint8 bg, fg;

	bg=14;
	fg=6;

	time[0]=(char)(0x30+(emu_minutes/10));
	time[1]=(char)(0x30+(emu_minutes%10));
	time[3]=(char)(0x30+(emu_seconds/10));
	time[4]=(char)(0x30+(emu_seconds%10));

	/*itoa_size(emu_minutes, time, 10, 3, '0');
	time[2]=':';
	itoa_size(emu_seconds, time+3, 10, 3, '0');*/
	text_out8(emu_screen, time, bg, fg, 0, 29, 0);

	text_out8(emu_screen, " speed ", bg, fg, 5, 29, 0);
	speed[0]=(char)(0x30+(emu_speed/100));
	speed[1]=(char)(0x30+((emu_speed%100)/10));
	speed[2]=(char)(0x30+((emu_speed%100)%10));
	text_out8(emu_screen, speed, bg, fg, 12, 29, 0);

	text_out8(emu_screen, " fps ", bg, fg, 16, 29, 0);
	fps[0]=(char)(0x30+((emu_fps%100)/10));
	fps[1]=(char)(0x30+((emu_fps%100)%10));
	text_out8(emu_screen, fps, bg, fg, 21, 29, 0);
	text_out8(emu_screen, "                 ", bg, fg, 23, 29, 0);

	if(drive8active) {
		bg=5;
		fg=6;
		text_out8(emu_screen, "(8)", bg, fg, 30, 29, 0);
	}
}

void draw_cursor() {
	static int is_moving=0;
	int is_moving_x, is_moving_y;
	int threshold=10;
	int stepping=3;

	cursor_x_last=cursor_x;
	if(button_state&LEFT_PRESSED) {
		if(is_moving==0||((is_moving>threshold)&&(is_moving%stepping==0))) {
			if(cursor_x>0) {
				cursor_x--;
			}
		}
		is_moving_x=1;
	} else if(button_state&RIGHT_PRESSED) {
		if(is_moving==0||((is_moving>threshold)&&(is_moving%stepping==0))) {
			if(cursor_x<39) {
				cursor_x++;
			}
		}
		is_moving_x=1;
	} else is_moving_x=0;
	
	cursor_y_last=cursor_y;
	if(button_state&UP_PRESSED) {
		if(is_moving==0||((is_moving>threshold)&&(is_moving%stepping==0))) {
			if(cursor_y>0) {
				cursor_y--;
			}
		}
		is_moving_y=1;
	} else if(button_state&DOWN_PRESSED) {
		if(is_moving==0||((is_moving>threshold)&&(is_moving%stepping==0))) {
			if(cursor_y<29) {
				cursor_y++;
			}
		}
		is_moving_y=1;
	} else is_moving_y=0;

	if(is_moving_x||is_moving_y) {
		is_moving++;
	} else {
		is_moving=0;
	}
	
	text_out8(emu_screen, "+", 0, 14, cursor_x, cursor_y, 1);
}

void draw_ui(C64 *TheC64) {
	if(status_enabled) draw_status();
	if(options_enabled) {
		draw_options(TheC64);
		return;
	}
	if(keyboard_enabled) {
		draw_keyboard(TheC64);
		draw_cursor();
	}
}

