
#ifndef _GP32_H_
#define _GP32_H_

#ifdef __cplusplus
extern "C" {
#endif

//--------------------------------------------------------------------
//-                  Program Status Register                         -
//- note: copied from agb header, might be right or not ;=P
//--------------------------------------------------------------------

#define PSR_CPU_MODE_MASK          0x1f        // CPU Mode

#define PSR_USER_MODE              0x10        // User
#define PSR_FIQ_MODE               0x11        // FIQ
#define PSR_IRQ_MODE               0x12        // IRQ
#define PSR_SVC_MODE               0x13        // Supervisor
#define PSR_ABORT_MODE             0x17        // Abort (Prefetch/Data)
#define PSR_UNDEF_MODE             0x1b        // Undefined Command
#define PSR_SYS_MODE               0x1f        // System

#define PSR_THUMB_STATE            0x20        // THUMB State
#define PSR_FIQ_DISABLE            0x40        // FIQ Disable
#define PSR_IRQ_DISABLE            0x80        // IRQ Disable
#define PSR_IRQ_FIQ_DISABLE        0xc0        // IRQ & FIQ Disable

#define PSR_V_FLAG                 0x10000000  // Overflow
#define PSR_C_FLAG                 0x20000000  // Carry/Borrow/Expand
#define PSR_Z_FLAG                 0x40000000  // Zero
#define PSR_N_FLAG                 0x80000000  // Negative/Less than

// Program Status Register Structure
typedef struct {
    unsigned long CpuMode:5;          // CPU Mode
    unsigned long Thumb_State:1;      // THUMB State
    unsigned long Fiq_Disable:1;      // FIQ Disable
    unsigned long Irq_Disable:1;      // IRQ Disable
    unsigned long Dummy_27_8:20;
    unsigned long V_Flag:1;           // Overflow
    unsigned long C_Flag:1;           // Carry/Borrow/Expand
    unsigned long Z_Flag:1;           // Zero
    unsigned long N_Flag:1;           // Negative/Less than
} PsrData;

typedef volatile PsrData vPsrData;


/*************************************************************
* Processor Clock speed		                                 *
*************************************************************/

#define CLK_SPEED_40    67800000    // 40 MHz
#define CLK_SPEED_60    59250000    // ???
#define CLK_SPEED_99    99000000    // 99Mhz - max!

/*

PLLCON Bit     Description           Initial State
MDIV   [19:12] Main divider control    0x5c/0x28
PDIV   [9:4]   Pre-divider control     0x08/0x08
SDIV   [1:0]   Post divider control     0x0/ 0x0

MPLLCON 0x14800004 R/W MPLL configuration register 0x0005c080
UPLLCON 0x14800008 R/W UPLL configuration register 0x00028080

NOTE: CLK_DIVN should be set carefully not to exceed the limit
      of HCLK & PCLK (the limit of HCLK & PCLK are 66MHz).
*/

#define CLK_NDIV_50    0x5c0a1    // ???
#define CLK_NDIV_60    0x47022    // ???
#define CLK_NDIV_99    0x3A002    // 99Mhz

/*
        HDIVN PDIVN FCLK HCLK   PCLK   Divide Ratio
          0     0   FCLK FCLK   FCLK   1 : 1 : 1
          0     1   FCLK FCLK   FCLK/2 1 : 1 : 2
          1     0   FCLK FCLK/2 FCLK/2 1 : 2 : 2
          1     1   FCLK FCLK/2 FCLK/4 1 : 2 : 4
*/
#define CLK_MODE_111    0
#define CLK_MODE_112    1
#define CLK_MODE_122    2
#define CLK_MODE_124    3

//#define FCLK (64800000) // ????
//#define HCLK (FCLK/1)   // 2?
//#define PCLK (FCLK/2)   // 4?

//#define FCLK 135428571
//#define HCLK (135428571/2)
//#define PCLK (135428571/4)

//#define FCLK (64800000) // ????
//#define HCLK (FCLK/1)   // 2?
//#define PCLK (FCLK/2)   // 4?

//--------------------------------------------------------------------
// hardware register
//--------------------------------------------------------------------

#define BOOT_ROM  0x00000000
#define WORK_RAM  0x0c000000

// MEMORY CONTROLLER

#define REG_BWSCON             0x14000000 // W  r/w   Bus Width & Wait Status Control
#define REG_BANKCON0           0x14000004 //          Boot ROM Control
#define REG_BANKCON1           0x14000008 //          BANK1 Control
#define REG_BANKCON2           0x1400000c //          BANK2 Control
#define REG_BANKCON3           0x14000010 //          BANK3 Control
#define REG_BANKCON4           0x14000014 //          BANK4 Control
#define REG_BANKCON5           0x14000018 //          BANK5 Control
#define REG_BANKCON6           0x1400001c //          BANK6 Control
#define REG_BANKCON7           0x14000020 //          BANK7 Control
#define REG_REFRESH            0x14000024 //          DRAM/SDRAM Refresh Control
#define REG_BANKSIZE           0x14000028 //          Flexible Bank Size
#define REG_MRSRB6             0x1400002c //          Mode register set for SDRAM
#define REG_MRSRB7             0x14000030 //          Mode register set for SDRAM

// USB HOST CONTROLLER

#define REG_HcRevision         0x14200000 // W        Control and Status Group
#define REG_HcControl          0x14200004 //
#define REG_HcCommonStatus     0x14200008 //
#define REG_HcInterruptStatus  0x1420000c //
#define REG_HcInterruptEnable  0x14200010 //
#define REG_HcInterruptDisable 0x14200014 //
#define REG_HcHCCA             0x14200018 //          Memory Pointer Group
#define REG_HcPeriodCuttentED  0x1420001c //
#define REG_HcControlHeadED    0x14200020 //
#define REG_HcControlCurrentED 0x14200024 //
#define REG_HcBulkHeadED       0x14200028 //
#define REG_HcBulkCurrentED    0x1420002c //
#define REG_HcDoneHead         0x14200030 //
#define REG_HcRmInterval       0x14200034 //          Frame Counter Group
#define REG_HcFmRemaining      0x14200038 //
#define REG_HcFmNumber         0x1420003c //
#define REG_HcPeriodicStart    0x14200040 //
#define REG_HcLSThreshold      0x14200044 //
#define REG_HcRhDescriptorA    0x14200048 //          Root Hub Group
#define REG_HcRhDescriptorB    0x1420004c //
#define REG_HcRhStatus         0x14200050 //
#define REG_HcRhPortStatus1    0x14200054 //
#define REG_HcRhPortStatus2    0x14200058 //

// INTERRUPT CONTROLLER

#define REG_SRCPND             0x14400000 // W  R/W   Interrupt Request Status
#define REG_INTMOD             0x14400004 // W        Interrupt Mode Control
#define REG_INTMSK             0x14400008 //    R/W   Interrupt Mask Control
#define REG_PRIORITY           0x1440000c // W        IRQ Priority Control
#define REG_INTPND             0x14400010 //    R/W   Interrupt Request Status
#define REG_INTOFFSET          0x14400014 //    R     Interrupt request source

// DMA

#define REG_DISRC0             0x14600000 // W  R/W   DMA 0 Initial Source
#define REG_DIDST0             0x14600004 //          DMA 0 Initial Destination
#define REG_DCON0              0x14600008 //          DMA 0 Control
#define REG_DSTAT0             0x1460000c //    R     DMA 0 Count
#define REG_DCSRC0             0x14600010 //          DMA 0 Current Source Address
#define REG_DCDST0             0x14600014 //          DMA 0 Current Destination Address
#define REG_DMASKTRIG0         0x14600018 //    R/W   DMA 0 Mask Trigger
#define REG_DISRC1             0x14600020 // W  R/W   DMA 1 Initial Source
#define REG_DIDST1             0x14600024 //          DMA 1 Initial Destination
#define REG_DCON1              0x14600028 //          DMA 1 Control
#define REG_DSTAT1             0x1460002c //    R     DMA 1 Count
#define REG_DCSRC1             0x14600030 //          DMA 1 Current Source Address
#define REG_DCDST1             0x14600034 //          DMA 1 Current Destination Address
#define REG_DMASKTRIG1         0x14600038 //    R/W   DMA 1 Mask Trigger
#define REG_DISRC2             0x14600040 // W  R/W   DMA 2 Initial Source
#define REG_DIDST2             0x14600044 //          DMA 2 Initial Destination
#define REG_DCON2              0x14600048 //          DMA 2 Control
#define REG_DSTAT2             0x1460004c //    R     DMA 2 Count
#define REG_DCSRC2             0x14600050 //          DMA 2 Current Source Address
#define REG_DCDST2             0x14600054 //          DMA 2 Current Destination Address
#define REG_DMASKTRIG2         0x14600058 //    R/W   DMA 2 Mask Trigger
#define REG_DISRC3             0x14600060 // W  R/W   DMA 3 Initial Source
#define REG_DIDST3             0x14600064 //          DMA 3 Initial Destination
#define REG_DCON3              0x14600068 //          DMA 3 Control
#define REG_DSTAT3             0x1460006c //    R     DMA 3 Count
#define REG_DCSRC3             0x14600060 //          DMA 3 Current Source Address
#define REG_DCDST3             0x14600064 //          DMA 3 Current Destination Address
#define REG_DMASKTRIG3         0x14600068 //    R/W   DMA 3 Mask Trigger

// CLOCK & POWER MANAGEMENT

#define REG_LOCKTIME           0x14800000 // W  R/W   PLL Lock Time Counter
#define REG_MPLLCON            0x14800004 //          MPLL Control
#define REG_UPLLCON            0x14800008 //          UPLL Control
#define REG_CLKCON             0x1480000c //          Clock Generator Control
#define REG_CLKSLOW            0x14800010 //          Slow Clock Control
#define REG_CLKDIVN            0x14800014 //          Clock divider Control

// LCD CONTROLLER

#define REG_LCDCON1            0x14a00000 // W  R/W   LCD Control 1
#define REG_LCDCON2            0x14a00004 //          LCD Control 2
#define REG_LCDCON3            0x14a00008 //          LCD Control 3
#define REG_LCDCON4            0x14a0000c //          LCD Control 4
#define REG_LCDCON5            0x14a00010 //          LCD Control 5
#define REG_LCDSADDR1          0x14a00014 //          STN/TFT: Frame Buffer Start Address1
#define REG_LCDSADDR2          0x14a00018 //          STN/TFT: Frame Buffer Start Address2
#define REG_LCDSADDR3          0x14a0001c //          STN/TFT: Virtual Screen Address Set
#define REG_REDLUT             0x14a00020 //          STN: Red Lookup Table
#define REG_GREENLUT           0x14a00024 //          STN: Green Lookup Table
#define REG_BLUELUT            0x14a00028 //          STN: Blue Lookup Table
#define REG_DITHMODE           0x14a0004c //          STN: Dithering Mode
#define REG_TPAL               0x14a00050 //          TFT: Temporary Palette

#define PALETTE_RAM            0x14A00400 //          palette memory

// UART

#define REG_ULCON0             0x15000000 // W  R/W   UART 0 Line Control
#define REG_UCON0              0x15000004 //          UART 0 Control
#define REG_UFCON0             0x15000008 //          UART 0 FIFO Control
#define REG_UMCON0             0x1500000c //          UART 0 Modem Control
#define REG_UTRSTAT0           0x15000010 //    R     UART 0 Tx/Rx Status
#define REG_UERSTAT0           0x15000014 //          UART 0 Rx Error Status
#define REG_UFSTAT0            0x15000018 //          UART 0 FIFO Status
#define REG_UMSTAT0            0x1500001c //          UART 0 Modem Status
#define REG_UTXH0              0x15000020 // B  W     UART 0 Transmission Hold
#define REG_URXH0              0x15000024 //    R     UART 0 Receive Buffer
#define REG_UBRDIV0            0x15000028 // W  R/W   UART 0 Baud Rate Divisor

#define REG_ULCON1             0x15004000 // W  R/W   UART 1 Line Control
#define REG_UCON1              0x15004004 //          UART 1 Control
#define REG_UFCON1             0x15004008 //          UART 1 FIFO Control
#define REG_UMCON1             0x1500400c //          UART 1 Modem Control
#define REG_UTRSTAT1           0x15004010 //    R     UART 1 Tx/Rx Status
#define REG_UERSTAT1           0x15004014 //          UART 1 Rx Error Status
#define REG_UFSTAT1            0x15004018 //          UART 1 FIFO Status
#define REG_UMSTAT1            0x1500401c //          UART 1 Modem Status
#define REG_UTXH1              0x15004020 // B  W     UART 1 Transmission Hold
#define REG_URXH1              0x15004024 //    R     UART 1 Receive Buffer
#define REG_UBRDIV1            0x15004028 // W  R/W   UART 1 Baud Rate Divisor

// PWM TIMER

#define REG_TCFG0              0x15100000 // W  R/W   Timer Configuration
#define REG_TCFG1              0x15100004 //          Timer Configuration
#define REG_TCON               0x15100008 //          Timer Control
#define REG_TCNTB0             0x1510000c //          Timer Count Buffer 0
#define REG_TCMPB0             0x15100010 //          Timer Compare Buffer 0
#define REG_TCNTO0             0x15100014 //    R     Timer Count Observation 0
#define REG_TCNTB1             0x15100018 //    R/W   Timer Count Buffer 1
#define REG_TCMPB1             0x1510001c //          Timer Compare Buffer 1
#define REG_TCNTO1             0x15100020 //    R     Timer Count Observation 1
#define REG_TCNTB2             0x15100024 //    R/W   Timer Count Buffer 2
#define REG_TCMPB2             0x15100028 //          Timer Compare Buffer 2
#define REG_TCNTO2             0x1510002c //    R     Timer Count Observation 2
#define REG_TCNTB3             0x15100030 //    R/W   Timer Count Buffer 3
#define REG_TCMPB3             0x15100034 //          Timer Compare Buffer 3
#define REG_TCNTO3             0x15100038 //    R     Timer Count Observation 3
#define REG_TCNTB4             0x1510003c //    R/W   Timer Count Buffer 4
#define REG_TCNTO4             0x15100040 //    R     Timer Count Observation 4

// USB DEVICE

#define REG_FUNC_ADDR_REG      0x15200140 // W  R/W   Function Address
#define REG_PWR_REG            0x15200144 //          Power Management
#define REG_INT_REG            0x15200148 //    R     Interrupt Pending and Clear
#define REG_INT_MASK_REG       0x1520014c //    R/W   Interrupt Mask
#define REG_FRAME_NUM_REG      0x15200150 //    R     Frame Number
#define REG_RESUME_CON_REG     0x15200154 //    R/W   Resume Signal Control
#define REG_EP0_CSR            0x15200160 //          Clock Generator Control
#define REG_EP0_MAXP           0x15200164 //          End Point0 MAX Packet
#define REG_EP0_OUT_CNT        0x15200168 //          End Point0 Out Write Count
#define REG_EP0_FIFO           0x1520016c //          End Point0 FIFO Read/Write
#define REG_EP1_IN_CSR         0x15200180 //          End Point1 in Control Status
#define REG_EP1_IN_MAXP        0x15200184 //          End Point1 in MAX Packet
#define REG_EP1_FIFO           0x15200188 // W        End Point2 FIFO Write
#define REG_EP2_IN_CSR         0x15200190 //    R/W   End Point2 in Control Status
#define REG_EP2_IN_MAXP        0x15200194 //          End Point2 in MAX Packet
#define REG_EP2_FIFO           0x15200198 // W        End Point2 FIFO Write
#define REG_EP3_OUT_CSR        0x152001a0 //    R/W   End Point3 Out Control Status
#define REG_EP3_OUT_MAXP       0x152001a4 //          End Point3 Out MAX Packet
#define REG_EP3_OUT_CNT        0x152001a8 //    R     End Point3 Out Write Count
#define REG_EP3_FIFO           0x152001ac //          End Point3 FIFO Read
#define REG_EP4_OUT_CSR        0x152001b0 //    R/W   End Point4 Out Control Status
#define REG_EP4_OUT_MAXP       0x152001b4 //          End Point4 Out MAX Packet
#define REG_EP4_OUT_CNT        0x152001b8 //    R     End Point4 Out Write Count
#define REG_EP4_FIFO           0x152001bc //          End Point4 FIFO Read
#define REG_DMA_CON            0x152001c0 //    R/W   DMA Interface Control
#define REG_DMA_UNIT           0x152001c4 //          DMA Transfer Unit Counter
#define REG_DMA_FIFO           0x152001c8 //          DMA Transfer FIFO Counter
#define REG_DMA_TX             0x152001cc //          DMA Total Transfer Counter
#define REG_TEST_MODE          0x152001f4 // W        Test Mode Control
#define REG_IN_CON_REG         0x152001f8 //          In Packet Number Control

// WATCHDOG TIMER

#define REG_WTCON              0x15300000 // W  R/W   Watch-Dog Timer Mode
#define REG_WTDAT              0x15300004 //          Watch-Dog Timer Data
#define REG_WTCNT              0x15300008 //          Watch-Dog Timer Count

// IIC

#define REG_IICCON             0x15400000 // W  R/W   IIC Control
#define REG_IICSTAT            0x15400004 //          IIC Status
#define REG_IICADD             0x15400008 //          IIC Address
#define REG_IICDS              0x1540000c //          IIC Data Shift

// IIS (Sound)

#define REG_IISCON             0x15508000 // HW,W R/W IIS Control
#define REG_IISMOD             0x15508004 // HW,W     IIS Mode
#define REG_IISPSR             0x15508008 // HW,W     IIS Prescaler
#define REG_IISFIFCON          0x1550800c // HW,W     IIS FIFO Control
#define REG_IISFIF             0x15508010 // HW       IIS FIFO Entry

// I/O PORT

#define REG_PACON              0x15600000 // W  R/W   Port A Control
#define REG_PADAT              0x15600004 //          Port A Data
#define REG_PBCON              0x15600008 //          Port B Control
#define REG_PBDAT              0x1560000c //          Port B Data
#define REG_PBUP               0x15600010 //          Pull-up Control B
#define REG_PCCON              0x15600014 //          Port C Control
#define REG_PCDAT              0x15600018 //          Port C Data
#define REG_PCUP               0x1560001c //          Pull-up Control C
#define REG_PDCON              0x15600020 //          Port D Control
#define REG_PDDAT              0x15600024 //          Port D Data
#define REG_PDUP               0x15600028 //          Pull-up Control D
#define REG_PECON              0x1560002c //          Port E Control
#define REG_PEDAT              0x15600030 //          Port E Data
#define REG_PEUP               0x15600034 //          Pull-up Control E
#define REG_PFCON              0x15600038 //          Port F Control
#define REG_PFDAT              0x1560003c //          Port F Data
#define REG_PFUP               0x15600040 //          Pull-up Control F
#define REG_PGCON              0x15600044 //          Port G Control
#define REG_PGDAT              0x15600048 //          Port G Data
#define REG_PGUP               0x1560004c //          Pull-up Control G
#define REG_OPENCR             0x15600050 //          Open Drain Enable
#define REG_MISCCR             0x15600054 //          Miscellaneous Control
#define REG_EXTINT             0x15600058 //          External Interrupt Control

// RTC

#define REG_RTCCON             0x15700040 // B  R/W   RTC Control
#define REG_TICINT             0x15700044 //          Tick time count
#define REG_RTCALM             0x15700050 //          RTC Alarm Control
#define REG_ALMSEC             0x15700054 //          Alarm Second
#define REG_ALMMIN             0x15700058 //          Alarm Minute
#define REG_ALMHOUR            0x1570005c //          Alarm Hour
#define REG_ALMDAY             0x15700060 //          Alarm Day
#define REG_ALMMON             0x15700064 //          Alarm Month
#define REG_ALMYEAR            0x15700068 //          Alarm Year
#define REG_RTCRST             0x1570006c //          RTC Round Reset
#define REG_BCDSEC             0x15700070 //          BCD Second
#define REG_BCDMIN             0x15700074 //          BCD Minute
#define REG_BCDHOUR            0x15700078 //          BCD Hour
#define REG_BCDDAY             0x1570007c //          BCD Day
#define REG_BCDDATE            0x15700080 //          BCD Date
#define REG_BCDMON             0x15700084 //          BCD Month
#define REG_BCDYEAR            0x15700088 //          BCD Year

// A/D CONVERTER

#define REG_ADCCON             0x15800000 // W  R/W   ADC Control
#define REG_ADCDAT             0x15800004 //    R     ADC Data

// SPI

#define REG_SPCON              0x15900000 // W  R/W   SPI Control
#define REG_SPSTA              0x15900004 //    R     SPI Status
#define REG_SPPIN              0x15900008 //    R/W   SPI Pin Control
#define REG_SPPRE              0x1590000c //          SPI Baud Rate Prescaler
#define REG_SPTDAT             0x15900010 //          SPI Tx Data
#define REG_SPRDAT             0x15900014 //    R     SPI Rx Data

// MMC INTERFACE

#define REG_MMCON              0x15a00000 // B,HW,W  R/W  MMC Control
#define REG_MMCRR              0x15a00004 //              MMC Command
#define REG_MMFCON             0x15a00008 //              MMC FIFO Control
#define REG_MMSTA              0x15a0000c //         R    MMC Status
#define REG_MMFSTA             0x15a00010 // HW,W         MMC FIFO Status
#define REG_MMPRE              0x15a00014 // B,HW,W  R/W  MMC Baud Rate Prescaler
#define REG_MMLEN              0x15a00018 // HW,W         MMC Block Length
#define REG_MMCR7              0x15a0001c // B,HW,W  R    Response CRC7
#define REG_MMRSP0             0x15a00020 // W            MMC Response Status 0
#define REG_MMRSP1             0x15a00024 //              MMC Response Status 1
#define REG_MMRSP2             0x15a00028 //              MMC Response Status 2
#define REG_MMRSP3             0x15a0002c //              MMC Response Status 3
#define REG_MMCMD0             0x15a00030 // B,HW,W  R/W  MMC Command 0
#define REG_MMCMD1             0x15a00034 // W            MMC Command 1
#define REG_MMCR16             0x15a00038 // HW,W    R    Data Read CRC16 Buffer
#define REG_MMDAT              0x15a0003c // B,HW,W  R/W  MMC Data

//--------------------------------------------------------------------
// headers
//--------------------------------------------------------------------

#include "24x.h"
#include "iis.h"
#include "rtc.h"

typedef unsigned long size_t;
//typedef unsigned long off_t;
typedef int off_t;
typedef int           pid_t;
//struct	stat
//{
/*
  dev_t		st_dev;
  ino_t		st_ino;
  mode_t	st_mode;
  nlink_t	st_nlink;
  uid_t		st_uid;
  gid_t		st_gid;
  dev_t		st_rdev;
*/
//  off_t		st_size;
//};
#include "syscall.h"

#include "graphics.h"

#include "align.h"
//#include "profile.h"
//#include "debug.h"

#include "gptypes.h"
//#include "conio.h"
/*
#include "stdlib/math.h"
#include "stdlib/rmath.h"
#include "stdlib/memory.h"
#include "stdlib/random.h"
#include "stdlib/string.h"
#include "stdlib/ctype.h"
*/

#ifdef __cplusplus
}      /* extern "C" */
#endif

#endif /* _GP32_H_ */

