
BOOT_ROM = 0x00000000
WORK_RAM = 0x0c000000

@--------------------------------------------------------------------
@-                  Program Status Register                         -
@- note: copied from agb header, might be right or not ;=P
@--------------------------------------------------------------------

PSR_CPU_MODE_MASK       =   0x1f        @ CPU Mode

PSR_USER_MODE           =   0x10        @ User
PSR_FIQ_MODE            =   0x11        @ FIQ
PSR_IRQ_MODE            =   0x12        @ IRQ
PSR_SVC_MODE            =   0x13        @ Supervisor
PSR_ABORT_MODE          =   0x17        @ Abort (Prefetch/Data)
PSR_UNDEF_MODE          =   0x1b        @ Undefined Command
PSR_SYS_MODE            =   0x1f        @ System

PSR_THUMB_STATE         =   0x20        @ THUMB State
PSR_FIQ_DISABLE         =   0x40        @ FIQ Disable
PSR_IRQ_DISABLE         =   0x80        @ IRQ Disable
PSR_IRQ_FIQ_DISABLE     =   0xc0        @ IRQ & FIQ Disable

PSR_V_FLAG              =   0x10000000  @ Overflow
PSR_C_FLAG              =   0x20000000  @ Carry/Borrow/Expand
PSR_Z_FLAG              =   0x40000000  @ Zero
PSR_N_FLAG              =   0x80000000  @ Negative/Less than

@--------------------------------------------------------------------
@ special hardware registers
@--------------------------------------------------------------------

@ MEMORY CONTROLLER

REG_BWSCON            = 0x14000000 @ W  r/w   Bus Width & Wait Status Control
REG_BANKCON0          = 0x14000004 @          Boot ROM Control
REG_BANKCON1          = 0x14000008 @          BANK1 Control
REG_BANKCON2          = 0x1400000c @          BANK2 Control
REG_BANKCON3          = 0x14000010 @          BANK3 Control
REG_BANKCON4          = 0x14000014 @          BANK4 Control
REG_BANKCON5          = 0x14000018 @          BANK5 Control
REG_BANKCON6          = 0x1400001c @          BANK6 Control
REG_BANKCON7          = 0x14000020 @          BANK7 Control
REG_REFRESH           = 0x14000024 @          DRAM/SDRAM Refresh Control
REG_BANKSIZE          = 0x14000028 @          Flexible Bank Size
REG_MRSRB6            = 0x1400002c @          Mode register set for SDRAM
REG_MRSRB7            = 0x14000030 @          Mode register set for SDRAM

@ USB HOST CONTROLLER

REG_HcRevision        = 0x14200000 @ W        Control and Status Group
REG_HcControl         = 0x14200004 @
REG_HcCommonStatus    = 0x14200008 @
REG_HcInterruptStatus = 0x1420000c @
REG_HcInterruptEnable = 0x14200010 @
REG_HcInterruptDisable= 0x14200014 @
REG_HcHCCA            = 0x14200018 @          Memory Pointer Group
REG_HcPeriodCuttentED = 0x1420001c @
REG_HcControlHeadED   = 0x14200020 @
REG_HcControlCurrentED= 0x14200024 @
REG_HcBulkHeadED      = 0x14200028 @
REG_HcBulkCurrentED   = 0x1420002c @
REG_HcDoneHead        = 0x14200030 @
REG_HcRmInterval      = 0x14200034 @          Frame Counter Group
REG_HcFmRemaining     = 0x14200038 @
REG_HcFmNumber        = 0x1420003c @
REG_HcPeriodicStart   = 0x14200040 @
REG_HcLSThreshold     = 0x14200044 @
REG_HcRhDescriptorA   = 0x14200048 @          Root Hub Group
REG_HcRhDescriptorB   = 0x1420004c @
REG_HcRhStatus        = 0x14200050 @
REG_HcRhPortStatus1   = 0x14200054 @
REG_HcRhPortStatus2   = 0x14200058 @

@ INTERRUPT CONTROLLER

REG_SRCPND            = 0x14400000 @ W  R/W   Interrupt Request Status
REG_INTMOD            = 0x14400004 @ W        Interrupt Mode Control
REG_INTMSK            = 0x14400008 @    R/W   Interrupt Mask Control
REG_PRIORITY          = 0x1440000c @ W        IRQ Priority Control
REG_INTPND            = 0x14400010 @    R/W   Interrupt Request Status
REG_INTOFFSET         = 0x14400014 @    R     Interrupt request source

@ DMA

REG_DISRC0            = 0x14600000 @ W  R/W   DMA 0 Initial Source
REG_DIDST0            = 0x14600004 @          DMA 0 Initial Destination
REG_DCON0             = 0x14600008 @          DMA 0 Control
REG_DSTAT0            = 0x1460000c @    R     DMA 0 Count
REG_DCSRC0            = 0x14600010 @          DMA 0 Current Source Address
REG_DCDST0            = 0x14600014 @          DMA 0 Current Destination Address
REG_DMASKTRIG0        = 0x14600018 @    R/W   DMA 0 Mask Trigger
REG_DISRC1            = 0x14600020 @ W  R/W   DMA 1 Initial Source
REG_DIDST1            = 0x14600024 @          DMA 1 Initial Destination
REG_DCON1             = 0x14600028 @          DMA 1 Control
REG_DSTAT1            = 0x1460002c @    R     DMA 1 Count
REG_DCSRC1            = 0x14600030 @          DMA 1 Current Source Address
REG_DCDST1            = 0x14600034 @          DMA 1 Current Destination Address
REG_DMASKTRIG1        = 0x14600038 @    R/W   DMA 1 Mask Trigger
REG_DISRC2            = 0x14600040 @ W  R/W   DMA 2 Initial Source
REG_DIDST2            = 0x14600044 @          DMA 2 Initial Destination
REG_DCON2             = 0x14600048 @          DMA 2 Control
REG_DSTAT2            = 0x1460004c @    R     DMA 2 Count
REG_DCSRC2            = 0x14600050 @          DMA 2 Current Source Address
REG_DCDST2            = 0x14600054 @          DMA 2 Current Destination Address
REG_DMASKTRIG2        = 0x14600058 @    R/W   DMA 2 Mask Trigger
REG_DISRC3            = 0x14600060 @ W  R/W   DMA 3 Initial Source
REG_DIDST3            = 0x14600064 @          DMA 3 Initial Destination
REG_DCON3             = 0x14600068 @          DMA 3 Control
REG_DSTAT3            = 0x1460006c @    R     DMA 3 Count
REG_DCSRC3            = 0x14600060 @          DMA 3 Current Source Address
REG_DCDST3            = 0x14600064 @          DMA 3 Current Destination Address
REG_DMASKTRIG3        = 0x14600068 @    R/W   DMA 3 Mask Trigger

@ CLOCK & POWER MANAGEMENT

REG_LOCKTIME          = 0x14800000 @ W  R/W   PLL Lock Time Counter
REG_MPLLCON           = 0x14800004 @          MPLL Control
REG_UPLLCON           = 0x14800008 @          UPLL Control
REG_CLKCON            = 0x1480000c @          Clock Generator Control
REG_CLKSLOW           = 0x14800010 @          Slow Clock Control
REG_CLKDIVN           = 0x14800014 @          Clock divider Control

@ LCD CONTROLLER

REG_LCDCON1           = 0x14a00000 @ W  R/W   LCD Control 1
REG_LCDCON2           = 0x14a00004 @          LCD Control 2
REG_LCDCON3           = 0x14a00008 @          LCD Control 3
REG_LCDCON4           = 0x14a0000c @          LCD Control 4
REG_LCDCON5           = 0x14a00010 @          LCD Control 5
REG_LCDSADDR1         = 0x14a00014 @          STN/TFT: Frame Buffer Start Address1
REG_LCDSADDR2         = 0x14a00018 @          STN/TFT: Frame Buffer Start Address2
REG_LCDSADDR3         = 0x14a0001c @          STN/TFT: Virtual Screen Address Set
REG_REDLUT            = 0x14a00020 @          STN: Red Lookup Table
REG_GREENLUT          = 0x14a00024 @          STN: Green Lookup Table
REG_BLUELUT           = 0x14a00028 @          STN: Blue Lookup Table
REG_DITHMODE          = 0x14a0004c @          STN: Dithering Mode
REG_TPAL              = 0x14a00050 @          TFT: Temporary Palette

PALETTE_RAM           = 0x14A00400 @          palette memory

@ UART

REG_ULCON0            = 0x15000000 @ W  R/W   UART 0 Line Control
REG_UCON0             = 0x15000004 @          UART 0 Control
REG_UFCON0            = 0x15000008 @          UART 0 FIFO Control
REG_UMCON0            = 0x1500000c @          UART 0 Modem Control
REG_UTRSTAT0          = 0x15000010 @    R     UART 0 Tx/Rx Status
REG_UERSTAT0          = 0x15000014 @          UART 0 Rx Error Status
REG_UFSTAT0           = 0x15000018 @          UART 0 FIFO Status
REG_UMSTAT0           = 0x1500001c @          UART 0 Modem Status
REG_UTXH0             = 0x15000020 @ B  W     UART 0 Transmission Hold
REG_URXH0             = 0x15000024 @    R     UART 0 Receive Buffer
REG_UBRDIV0           = 0x15000028 @ W  R/W   UART 0 Baud Rate Divisor

REG_ULCON1            = 0x15004000 @ W  R/W   UART 1 Line Control
REG_UCON1             = 0x15004004 @          UART 1 Control
REG_UFCON1            = 0x15004008 @          UART 1 FIFO Control
REG_UMCON1            = 0x1500400c @          UART 1 Modem Control
REG_UTRSTAT1          = 0x15004010 @    R     UART 1 Tx/Rx Status
REG_UERSTAT1          = 0x15004014 @          UART 1 Rx Error Status
REG_UFSTAT1           = 0x15004018 @          UART 1 FIFO Status
REG_UMSTAT1           = 0x1500401c @          UART 1 Modem Status
REG_UTXH1             = 0x15004020 @ B  W     UART 1 Transmission Hold
REG_URXH1             = 0x15004024 @    R     UART 1 Receive Buffer
REG_UBRDIV1           = 0x15004028 @ W  R/W   UART 1 Baud Rate Divisor

@ PWM TIMER

REG_TCFG0             = 0x15100000 @ W  R/W   Timer Configuration
REG_TCFG1             = 0x15100004 @          Timer Configuration
REG_TCON              = 0x15100008 @          Timer Control
REG_TCNTB0            = 0x1510000c @          Timer Count Buffer 0
REG_TCMPB0            = 0x15100010 @          Timer Compare Buffer 0
REG_TCNTO0            = 0x15100014 @    R     Timer Count Observation 0
REG_TCNTB1            = 0x15100018 @    R/W   Timer Count Buffer 1
REG_TCMPB1            = 0x1510001c @          Timer Compare Buffer 1
REG_TCNTO1            = 0x15100020 @    R     Timer Count Observation 1
REG_TCNTB2            = 0x15100024 @    R/W   Timer Count Buffer 2
REG_TCMPB2            = 0x15100028 @          Timer Compare Buffer 2
REG_TCNTO2            = 0x1510002c @    R     Timer Count Observation 2
REG_TCNTB3            = 0x15100030 @    R/W   Timer Count Buffer 3
REG_TCMPB3            = 0x15100034 @          Timer Compare Buffer 3
REG_TCNTO3            = 0x15100038 @    R     Timer Count Observation 3
REG_TCNTB4            = 0x1510003c @    R/W   Timer Count Buffer 4
REG_TCNTO4            = 0x15100040 @    R     Timer Count Observation 4

@ USB DEVICE

REG_FUNC_ADDR_REG     = 0x15200140 @ W  R/W   Function Address
REG_PWR_REG           = 0x15200144 @          Power Management
REG_INT_REG           = 0x15200148 @    R     Interrupt Pending and Clear
REG_INT_MASK_REG      = 0x1520014c @    R/W   Interrupt Mask
REG_FRAME_NUM_REG     = 0x15200150 @    R     Frame Number
REG_RESUME_CON_REG    = 0x15200154 @    R/W   Resume Signal Control
REG_EP0_CSR           = 0x15200160 @          Clock Generator Control
REG_EP0_MAXP          = 0x15200164 @          End Point0 MAX Packet
REG_EP0_OUT_CNT       = 0x15200168 @          End Point0 Out Write Count
REG_EP0_FIFO          = 0x1520016c @          End Point0 FIFO Read/Write
REG_EP1_IN_CSR        = 0x15200180 @          End Point1 in Control Status
REG_EP1_IN_MAXP       = 0x15200184 @          End Point1 in MAX Packet
REG_EP1_FIFO          = 0x15200188 @ W        End Point2 FIFO Write
REG_EP2_IN_CSR        = 0x15200190 @    R/W   End Point2 in Control Status
REG_EP2_IN_MAXP       = 0x15200194 @          End Point2 in MAX Packet
REG_EP2_FIFO          = 0x15200198 @ W        End Point2 FIFO Write
REG_EP3_OUT_CSR       = 0x152001a0 @    R/W   End Point3 Out Control Status
REG_EP3_OUT_MAXP      = 0x152001a4 @          End Point3 Out MAX Packet
REG_EP3_OUT_CNT       = 0x152001a8 @    R     End Point3 Out Write Count
REG_EP3_FIFO          = 0x152001ac @          End Point3 FIFO Read
REG_EP4_OUT_CSR       = 0x152001b0 @    R/W   End Point4 Out Control Status
REG_EP4_OUT_MAXP      = 0x152001b4 @          End Point4 Out MAX Packet
REG_EP4_OUT_CNT       = 0x152001b8 @    R     End Point4 Out Write Count
REG_EP4_FIFO          = 0x152001bc @          End Point4 FIFO Read
REG_DMA_CON           = 0x152001c0 @    R/W   DMA Interface Control
REG_DMA_UNIT          = 0x152001c4 @          DMA Transfer Unit Counter
REG_DMA_FIFO          = 0x152001c8 @          DMA Transfer FIFO Counter
REG_DMA_TX            = 0x152001cc @          DMA Total Transfer Counter
REG_TEST_MODE         = 0x152001f4 @ W        Test Mode Control
REG_IN_CON_REG        = 0x152001f8 @          In Packet Number Control

@ WATCHDOG TIMER

REG_WTCON             = 0x15300000 @ W  R/W   Watch-Dog Timer Mode
REG_WTDAT             = 0x15300004 @          Watch-Dog Timer Data
REG_WTCNT             = 0x15300008 @          Watch-Dog Timer Count

@ IIC

REG_IICCON            = 0x15400000 @ W  R/W   IIC Control
REG_IICSTAT           = 0x15400004 @          IIC Status
REG_IICADD            = 0x15400008 @          IIC Address
REG_IICDS             = 0x1540000c @          IIC Data Shift

@ IIS

REG_IISCON            = 0x15508000 @ HW,W R/W IIS Control
REG_IISMOD            = 0x15508004 @ HW,W     IIS Mode
REG_IISPSR            = 0x15508008 @ HW,W     IIS Prescaler
REG_IISFIFCON         = 0x1550800c @ HW,W     IIS FIFO Control
REG_IISFIF            = 0x15508010 @ HW       IIS FIFO Entry

@ I/O PORT

REG_PACON             = 0x15600000 @ W  R/W   Port A Control
REG_PADAT             = 0x15600004 @          Port A Data
REG_PBCON             = 0x15600008 @          Port B Control
REG_PBDAT             = 0x1560000c @          Port B Data
REG_PBUP              = 0x15600010 @          Pull-up Control B
REG_PCCON             = 0x15600014 @          Port C Control
REG_PCDAT             = 0x15600018 @          Port C Data
REG_PCUP              = 0x1560001c @          Pull-up Control C
REG_PDCON             = 0x15600020 @          Port D Control
REG_PDDAT             = 0x15600024 @          Port D Data
REG_PDUP              = 0x15600028 @          Pull-up Control D
REG_PECON             = 0x1560002c @          Port E Control
REG_PEDAT             = 0x15600030 @          Port E Data
REG_PEUP              = 0x15600034 @          Pull-up Control E
REG_PFCON             = 0x15600038 @          Port F Control
REG_PFDAT             = 0x1560003c @          Port F Data
REG_PFUP              = 0x15600040 @          Pull-up Control F
REG_PGCON             = 0x15600044 @          Port G Control
REG_PGDAT             = 0x15600048 @          Port G Data
REG_PGUP              = 0x1560004c @          Pull-up Control G
REG_OPENCR            = 0x15600050 @          Open Drain Enable
REG_MISCCR            = 0x15600054 @          Miscellaneous Control
REG_EXTINT            = 0x15600058 @          External Interrupt Control

@ RTC

REG_RTCCON            = 0x15700040 @ B  R/W   RTC Control
REG_TICINT            = 0x15700044 @          Tick time count
REG_RTCALM            = 0x15700050 @          RTC Alarm Control
REG_ALMSEC            = 0x15700054 @          Alarm Second
REG_ALMMIN            = 0x15700058 @          Alarm Minute
REG_ALMHOUR           = 0x1570005c @          Alarm Hour
REG_ALMDAY            = 0x15700060 @          Alarm Day
REG_ALMMON            = 0x15700064 @          Alarm Month
REG_ALMYEAR           = 0x15700068 @          Alarm Year
REG_RTCRST            = 0x1570006c @          RTC Round Reset
REG_BCDSEC            = 0x15700070 @          BCD Second
REG_BCDMIN            = 0x15700074 @          BCD Minute
REG_BCDHOUR           = 0x15700078 @          BCD Hour
REG_BCDDAY            = 0x1570007c @          BCD Day
REG_BCDDATE           = 0x15700080 @          BCD Date
REG_BCDMON            = 0x15700084 @          BCD Month
REG_BCDYEAR           = 0x15700088 @          BCD Year

@ A/D CONVERTER

REG_ADCCON            = 0x15800000 @ W  R/W   ADC Control
REG_ADCDAT            = 0x15800004 @    R     ADC Data

@ SPI

REG_SPCON             = 0x15900000 @ W  R/W   SPI Control
REG_SPSTA             = 0x15900004 @    R     SPI Status
REG_SPPIN             = 0x15900008 @    R/W   SPI Pin Control
REG_SPPRE             = 0x1590000c @          SPI Baud Rate Prescaler
REG_SPTDAT            = 0x15900010 @          SPI Tx Data
REG_SPRDAT            = 0x15900014 @    R     SPI Rx Data

@ MMC INTERFACE

REG_MMCON             = 0x15a00000 @ B,HW,W  R/W  MMC Control
REG_MMCRR             = 0x15a00004 @              MMC Command
REG_MMFCON            = 0x15a00008 @              MMC FIFO Control
REG_MMSTA             = 0x15a0000c @         R    MMC Status
REG_MMFSTA            = 0x15a00010 @ HW,W         MMC FIFO Status
REG_MMPRE             = 0x15a00014 @ B,HW,W  R/W  MMC Baud Rate Prescaler
REG_MMLEN             = 0x15a00018 @ HW,W         MMC Block Length
REG_MMCR7             = 0x15a0001c @ B,HW,W  R    Response CRC7
REG_MMRSP0            = 0x15a00020 @ W            MMC Response Status 0
REG_MMRSP1            = 0x15a00024 @              MMC Response Status 1
REG_MMRSP2            = 0x15a00028 @              MMC Response Status 2
REG_MMRSP3            = 0x15a0002c @              MMC Response Status 3
REG_MMCMD0            = 0x15a00030 @ B,HW,W  R/W  MMC Command 0
REG_MMCMD1            = 0x15a00034 @ W            MMC Command 1
REG_MMCR16            = 0x15a00038 @ HW,W    R    Data Read CRC16 Buffer
REG_MMDAT             = 0x15a0003c @ B,HW,W  R/W  MMC Data

.include "syscall.inc"
@.include "6502macro.inc"
