
#include <gp32.h>

#define L3DELAY                            16

/* some internal variables */

//unsigned char *Buf;

unsigned int size=0;
unsigned int fs=0;

unsigned char iis_bytespertransfer=0;
unsigned char iis_stereoflag=0;
unsigned char iis_bitcount=0;

/*
        L3-bus routines
*/

/*

Data transfer of type status

BIT 7 BIT 6 BIT 5 BIT 4 BIT 3 BIT 2 BIT 1 BIT 0  REGISTER

   0     X   SC1   SC0   IF2   IF1   IF0    DC   System Clock frequency (1 : 0)
                                                 data Input Format (2 : 0)
                                                 DC-filter
   1     X     X     X     X     X     X     X   not used

Data transfer of type data

BIT 7 BIT 6 BIT 5 BIT 4 BIT 3 BIT 2 BIT 1 BIT 0  REGISTER

   0     0   VC5   VC4   VC3   VC2   VC1   VC0   Volume Control (5 : 0)
   0     1   BB3   BB2   BB1   BB0   TR1   TR0   Bass Boost (3 : 0)
                                                 Treble (1 : 0)
   1     0     X   DE1   DE0    MT    M1    M0   DE-emphasis (1 : 0)
                                                 MuTe
                                                 Mode (1 : 0)
   1     1     X     X     X     X   PC1   PC0   Power Control (1 : 0)

*/

void _WrL3Addr(unsigned char data)
{
    signed long i,j;

    rPEDAT = rPEDAT & ~L3_MASK;       //L3D=L/L3M=L(in address mode)/L3C=L
    rPEDAT |= L3C;                    //L3C=H
    for(j=0;j<L3DELAY;j++);           //tsu(L3) > 190ns

    //PD[8:6]=L3D:L3C:L3M
    for(i=0;i<8;i++)	//LSB first
    {
	if(data&0x1)	//if data's LSB is 'H'
	{
            rPEDAT &= ~L3C;           //L3C=L
            rPEDAT |= L3D;            //L3D=H
            for(j=0;j<L3DELAY;j++);   //tcy(L3) > 500ns
            rPEDAT |= L3C;            //L3C=H
            rPEDAT |= L3D;            //L3D=H
            for(j=0;j<L3DELAY;j++);   //tcy(L3) > 500ns
	}
	else		//if data's LSB is 'L'
	{
            rPEDAT &= ~L3C;           //L3C=L
            rPEDAT &= ~L3D;           //L3D=L
            for(j=0;j<L3DELAY;j++);   //tcy(L3) > 500ns
            rPEDAT |= L3C;            //L3C=H
            rPEDAT &= ~L3D;           //L3D=L
            for(j=0;j<L3DELAY;j++);   //tcy(L3) > 500ns
	}
	data >>=1;
    }
    //L3M=H,L3C=H
    rPEDAT = (rPEDAT & ~L3_MASK) | (L3C|L3M);
}

void _WrL3Data(unsigned char data,int halt)
{
    signed long i,j;

    if(halt) {

        rPEDAT |= L3C;                //L3C=H(while tstp, L3 interface halt condition)
        for(j=0;j<L3DELAY;j++);       //tstp(L3) > 190ns
        rPEDAT &= (~L3M);
        for (j=0;j<L3DELAY;j++);
        rPEDAT |= L3M;
    }

    rPEDAT = (rPEDAT & ~L3_MASK) | (L3C|L3M);        //L3M=H(in data transfer mode)
    for(j=0;j<L3DELAY;j++);                          //tsu(L3)D > 190ns

    //PD[8:6]=L3D:L3C:L3M
    for(i=0;i<8;i++)
    {
        if(data&0x1)	//if data's LSB is 'H'
        {
            rPEDAT &= ~L3C;           //L3C=L
            rPEDAT |= L3D;            //L3D=H
            for(j=0;j<L3DELAY;j++);   //tcy(L3) > 500ns
            rPEDAT |= (L3C|L3D);      //L3C=H,L3D=H
            for(j=0;j<L3DELAY;j++);   //tcy(L3) > 500ns
        }
        else		//if data's LSB is 'L'
        {
            rPEDAT &= ~L3C;           //L3C=L
            rPEDAT &= ~L3D;           //L3D=L
            for(j=0;j<L3DELAY;j++);   //tcy(L3) > 500ns
            rPEDAT |= L3C;            //L3C=H
            rPEDAT &= ~L3D;           //L3D=L
            for(j=0;j<L3DELAY;j++);   //tcy(L3) > 500ns
        }
        data>>=1;	//for check next bit
    }

    rPEDAT = (rPEDAT & ~L3_MASK) | L3C;
    for (j=0;j<L3DELAY;j++);
    rPEDAT |= L3M;

}
