
#ifndef __IIS_H__
#define __IIS_H__

#ifdef __cplusplus
extern "C" {
#endif

#define GPC_EPCM_OK             0
#define GPC_EPCM_FULL		1
#define GPC_EPCM_NO_INIT	2

typedef enum{
	PCM_M11,
	PCM_S11,
	PCM_M22,
	PCM_S22,
	PCM_M44,
	PCM_S44
}PCM_SR;
typedef enum{
        PCM_8BIT        =0,
        PCM_16BIT       =1
}PCM_BIT;

/* iis/l3 routines / (used internally) */

#define L3C                             0x200   //bit 9
#define L3M                             0x400   //bit 10
#define L3D                             0x800   //bit 11
#define L3_MASK				0xe00

void _WrL3Addr(unsigned char data);
void _WrL3Data(unsigned char data,int halt);

/* initialize uda1330a by sending appropriate commands over L3 bus
fsflag  00  : 512,384,256fs         (SC : System Clock Freq)
type   000,  : iis,lsb16,lsb18,lsb20,msb                        */
void Init1330(int fsflag,int type);

/* setup IIS bus
	bits                  8 / 16 bit per channel
	type                  iis compatible samples / MSB samples
	mclk                  master clock    256,384fs
	bclk                  serial bitclock 16,32,48fs
	prescale a,b
	- higher numbers for a and b mean slower playing sample
*/
void InitIIS(int bits,int type,int mclk,int bclk,int prea,int preb);

int GpPcmInit(PCM_SR sr, PCM_BIT bit_count);

void GpControlVolume(int vol);

int GpPcmPlay(unsigned short *src, int size, int repeatflag);
void GpPcmStop(void);

extern unsigned char *Buf;

extern unsigned int size;
extern unsigned int fs;

extern unsigned char iis_bytespertransfer;
extern unsigned char iis_stereoflag;
extern unsigned char iis_bitcount;

#ifdef __cplusplus
}
#endif

#endif /*__IIS_H__*/
