
/********************************************************************/
/*            GP32 System Calls                                      */
/********************************************************************/

#ifndef _GP32_SYSTEM_CALL_H_
#define _GP32_SYSTEM_CALL_H_

#ifdef __cplusplus
extern "C" {
#endif

#define SystemCall(No)   asm ("SWI "#No)

/*------------------------------------------------------------------*/
/*                   System Call No.                                */
/*------------------------------------------------------------------*/

#define	_SWI_SWI_ASSERT              0x00
#define	_SWI_SWI_INT_INIT            0x01
#define	_SWI_MMU_SET                 0x02
#define	_SWI_CACHE_FLUSH             0x03
#define	_SWI_SW_RESET                0x04
#define	_SWI_EXE_PROGRAM             0x05
#define	_SWI_EXE_RO_COPY             0x06
#define	_SWI_PORT_CONFIG             0x07
#define	_SWI_LCD_STATUS              0x08
#define	_SWI_INSTALL_IRQ             0x09
#define	_SWI_UNINSTALL_IRQ   		 0x0a
#define	_SWI_GET_BASE                0x0b
#define	_SWI_GET_BASE_SIZE           0x0c
#define	_SWI_CLOCK_OPER              0x0d
#define	_SWI_SET_SNDBUFFER           0x0e
#define	_SWI_APPVAR_ACCESS           0x0f
#define	_SWI_KEY_SCAN                0x10
#define	_SWI_CHK_SMC                 0x11
#define	_SWI_USBD_FUNC               0x12
#define	_SWI_TMR_OPER                0x13
#define	_SWI_IIS_OPER                0x14
#define	_SWI_APP_ARGUMENT            0x15
#define	_SWI_PAL_OPER                0x16
#define	_SWI_VOL_CONTROL             0x17

#define	_SWI_COMM_RESET        		0x101
#define	_SWI_COMM_DELETE       		0x102
#define	_SWI_SMC_ID_GET        		0x103
#define	_SWI_DEV_ID_GET        		0x104
#define	_SWI_E2PROM_ACCESS     		0x105
#define	_SWI_DBG_INIT          		0x1ff

/*------------------------------------------------------------------*/
/*                   System Functions                               */
/*------------------------------------------------------------------*/

// 0x02
//void swi_mmu_change(r0, r1, r2);

// 0x04
void GpAppExit(void);
// 0x05
int GpAppExecute(char * p_code_ptr, const char * s_path);
//;*int GpUserInfoGet(char * p_id, char * p_pwd)
//;*int GpArgSet(int len, char * p_arg);
//;char * GpAppPathGet(int * n_len);
//;int GpAppPathSet(const char * p_name, int n_len);

// 0x07
//swi_port_config_reset

// 0x08
int GpGraphicModeSet(unsigned long gd_bpp, unsigned long* gp_pal);

//#ifdef GPDRAWSURFACE // also in graphics.h
//#undef GPDRAWSURFACE
//#endif
typedef struct {
	    unsigned char* ptbuffer; // buffer start address (aligned!)
        int bpp;                 // reserved
        int buf_w;               // buffer width
        int buf_h;               // buffer height
        int ox;                  // buffer origin x co-ordinate
        int oy;                  // buffer origin y co-ordinate
	    unsigned char* o_buffer; // buffer start address (original, non aligned)
} GPDRAWSURFACE;
#define GPC_DFLAG_1BPP	1
#define GPC_DFLAG_2BPP	2
#define GPC_DFLAG_4BPP	4
#define GPC_DFLAG_8BPP	8
#define GPC_DFLAG_16BPP	16

int GpLcdSurfaceGet(GPDRAWSURFACE * ptgpds, int idx);
//unsigned long GpSurfaceCreate (GPDRAWSURFACE* ptgpds, int gdflag); ?!?!? gibbet nix !?

void GpLcdEnable(void);
void GpLcdDisable(void);

#define GPC_FLIP_SLOW	0
#define GPC_FLIP_MID	1
#define GPC_FLIP_FAST	2
#define GPC_FLIP_DEFAULT	GPC_FLIP_MID

void GpFlipModeSet(int mode);
//;unsigned char * GpLcdLock(void);
//;int GpLcdStatusGet(void)

typedef struct tagGPLCDINFO{
	union{
		int U32_lcd;
		struct{
			char clk_value;
			char lcd_buf_count;
			char bpp;
			char b_lcd_on;
		}U8_lcd;
	}lcd_global;
	unsigned int buffer_size;
	unsigned int * buf_addr[4];
	unsigned int * r_palette;
	unsigned int * m_palette;
}GPLCDINFO;

void GpLcdInfoGet(GPLCDINFO * p_info);

// 0x09
void swi_install_irq(unsigned long nr,void *ptr);
// 0x0a
void swi_uninstall_irq(unsigned long nr);

// 0x0b
typedef void (*tGpSurfaceSet)(GPDRAWSURFACE* ptgpds);
//typedef void (*tGpSurfaceSet);
extern tGpSurfaceSet GpSurfaceSet;
extern tGpSurfaceSet GpSurfaceFlip;
tGpSurfaceSet* Init(void);

// 0x0b r0=2
//;void swi_pal_addr_get(&ptr, &reg)

// 0x0b r0=3
//;void swi_io_oper_set(unsigned int** ptr)

// 0x0b r0=4    get clock info (returns pointer to 3 longs in r0)
unsigned long GpMClkGet(void);
unsigned long GpHClkGet(void);
unsigned long GpPClkGet(void);

// 0x0b r0=5    get heap end (returned in r0)
// 0x0b r0=6    get time passed (returned in r0)

// 0x0d
int GpClockSpeedChange(int master_speed, int div_factor, int clk_mode);

// 0x0e
//swi_set_sndbuffer
//		;r0 = unsigned short * _sndmixedbuf[2]
//		;r1 = size
//extern void swi_set_sndbuffer(unsigned short **, int);

// 0x0f - environment
int GpAppPathSet(const char * p_path, int n_len);
char * GpAppPathGet(int * n_len /* optional */);
int GpUserInfoGet(char * p_id, char * p_pwd);
int GpArgSet(int len, char * p_arg);

//key definition
#define	GPC_VK_NONE		0x00
#define	GPC_VK_LEFT		0x01
#define	GPC_VK_UP		0x08
#define	GPC_VK_RIGHT	0x04
#define	GPC_VK_DOWN		0x02

#define GPC_VK_F1		0x80
#define GPC_VK_F2		0x10
#define GPC_VK_F3		0x20
#define GPC_VK_ENTER	0x40

#define GPC_VK_START	0x100
#define GPC_VK_SELECT	0x200

#define GPC_VK_FA		GPC_VK_ENTER
#define GPC_VK_FB		GPC_VK_F3
#define GPC_VK_FL		GPC_VK_F2
#define GPC_VK_FR		GPC_VK_F1

// 0x10 r0:0
void GpKeyInit(void);
// 0x10
unsigned long GpKeyGet(void);

// 0x11 check smc
//;int swi_smc_in(void)

// 0x12		usb functions
// 0x12 	clean up after reset (?)
//;void swi_usb_sof_get
//;int swi_usb_sof_get

struct tag_timer_reg{
	int cmd;		//0 - all stop/disable, (1 - setting | 2 - start), 4 - pause, 8 - stop
	int val_timer;
	unsigned int evt_handler;
	int val_freq;
	int val_tps;
};
// 0x13 r0: ptr to timer info
int swi_timer_operate(struct tag_timer_reg *p_tmr);

// 0x14
// int swi_iis_operate(struct tag_iis_info * p_iis)
void swi_iis_operate(unsigned long parm);

// 0x15		get command line arguments (returns *argv[] in r0 and argc r1)

#define GPC_PAL_RED_MASK	0xf800
#define GPC_PAL_GREEN_MASK	0x07c0
#define GPC_PAL_BLUE_MASK	0x003e
#define GPC_PALCHAN_RED_BIT		0x4
#define GPC_PALCHAN_GREEN_BIT	0x2
#define GPC_PALCHAN_BLUE_BIT	0x1

typedef unsigned short GP_PALETTEENTRY;
typedef struct tag_GP_LOGPALENTRY{
	unsigned char peRed;
	unsigned char peGreen;
	unsigned char peBlue;
	unsigned char peFlags;
}GP_LOGPALENTRY;
typedef unsigned long * GP_HPALETTE;

// 0x16
int swi_pal_oper(int , GP_HPALETTE );
int swi_chan_pal_fade(int , int , GP_HPALETTE );
void swi_pal_addr_get(GP_HPALETTE *ptr, GP_HPALETTE *srcptr);


// 0x17  broken !
//void GpControlVolume(unsigned long parm1);

// 0x101
//	;int GpCommCreate(GPN_DESC * p_desc, GPN_COMM * p_comm)
// 0x102
//	;void GpCommDelete(GPN_DESC * p_desc);

// these aren't swi's but fit best here anyway
void ARMEnableInterrupt(void);
void ARMDisableInterrupt(void);

#ifdef __cplusplus
}      /* extern "C" */
#endif

#endif /* _GP32_SYSTEM_CALL_H */
