
@********************************************************************
@*            GP32 system calls                                     *
@********************************************************************

                .INCLUDE "gp32.inc"

@********************************************************************
@ SWI 0x04,0x05 - Application Execute/Exit
@********************************************************************

@ void GpAppExit(void);

		.TEXT
                .CODE 32
                .align 4

                .global GpAppExit
GpAppExit:
                STMDB    r13!,{r1-r12,r14}
                SWI      0x4
                LDMIA    r13!,{r1-r12,pc}

@ int GpAppExecute(char * p_code_ptr, const char * s_path);
@
@                 .TEXT
@                 .CODE 32
@                 .align 4
@
@                 .global GpAppExecute
@ GpAppExecute:
@                 STMDB    r13!,{r1-r12,r14}
@                 SWI      0x5
@                 LDMIA    r13!,{r1-r12,pc}

@********************************************************************
@ SWI 8 - LCD Services
@********************************************************************

		.TEXT
                .CODE 32
                .align 4

                .global GpSurfaceSet
                .global GpSurfaceFlip

GpSurfaceSet:   .word 0x00000000
GpSurfaceFlip:  .word 0x00000000

@ int GpGraphicModeSet(int gd_bpp, int * gp_pal);

		.TEXT
                .CODE 32
                .align 4

                .global GpGraphicModeSet
GpGraphicModeSet:
                STMDB sp!,{r2,lr}
                mov r2, #0x00       @ sub function 0
                swi 0x8             @ {r0-r2}, {r0}
                LDMIA sp!,{r2,pc}

@ int GpLcdSurfaceGet(GPDRAWSURFACE * ptgpds, int idx)
@ r0=pointer to surface

		.TEXT
                .CODE 32
                .align 4

                .global GpLcdSurfaceGet
GpLcdSurfaceGet:
                STMDB sp!,{r2,lr}
                mov r2, #0x01       @ sub function 1
                swi 0x8             @ {r0-r2}, {r0}
                LDMIA sp!,{r2,pc}

                .global GpFlipModeSet
GpFlipModeSet:
                STMDB    r13!,{r1,r2,r14}
                MOV      r2,#2
                SWI      0x8
                LDMIA    r13!,{r1,r2,pc}

@ turns lcd on
@ void GpLcdEnable();

		.TEXT
                .CODE 32
                .align 4

                .global GpLcdEnable
GpLcdEnable:
                STMDB sp!,{r2,lr}
                mov r2, #0x03       @ sub function 3
                swi 0x8             @ {r2}
                LDMIA sp!,{r2,pc}

		.TEXT
                .CODE 32
                .align 4

@ turns lcd off
@ void GpLcdDisable();

                .global GpLcdDisable
GpLcdDisable:
                STMDB sp!,{r2,lr}
                mov r2, #0x04       @ sub function 4
                swi 0x8             @ {r2}
                LDMIA sp!,{r2,pc}

                .global GpLcdInfoGet
GpLcdInfoGet:
                STMDB    r13!,{r2,r14}
                MOV      r2,#7         @ sub function 7
                SWI      0x8
                LDR      r2,[r13],#4
                LDR      r14,[r13],#4
                MOV      pc,r14

@********************************************************************
@ SWI 0xo9,0x0a - irq
@********************************************************************

@                .global swi_install_irq
@ swi_install_irq:
@                 STMDB    r13!,{r14}
@                 SWI      0x9
@                 LDMIA    r13!,{pc}
@
@                 .global swi_uninstall_irq
@ swi_uninstall_irq:
@                 STMDB    r13!,{r14}
@                 SWI      0xa
@                 LDMIA    r13!,{pc}

@********************************************************************
@ SWI 0xb - Init
@********************************************************************

@ gets entry point to gpSurfaceSet/gpSurfaceFlip
@ void Init();

		.TEXT
                .CODE 32
                .align 4

                .global Init
Init:
                STMDB sp!,{lr}
                mov r0, #0x00       @ sub functiom 0
                swi 0xb
                LDMIA sp!,{pc}

                .global swi_pal_addr_get
swi_pal_addr_get:
                STMDB    r13!,{r2,r3,r14}
                MOV      r2,r1
                MOV      r1,r0
                MOV      r0,#2      @ sub functiom 2
                SWI      0xb
                LDMIA    r13!,{r2,r3,pc}

                .global GpMClkGet
GpMClkGet:
                STMDB    r13!,{r14}
                MOV      r0,#4      @ sub functiom 4
                SWI      0xb
                LDR      r0,[r0,#0]
                LDMIA    r13!,{pc}

                .global GpHClkGet
GpHClkGet:
                STMDB    r13!,{r14}
                MOV      r0,#4      @ sub functiom 4
                SWI      0xb
                LDR      r0,[r0,#4]
                LDMIA    r13!,{pc}

                .global GpPClkGet
GpPClkGet:
                STMDB    r13!,{r14}
                MOV      r0,#4      @ sub functiom 4
                SWI      0xb
                LDR      r0,[r0,#8]
                LDMIA    r13!,{pc}


@********************************************************************
@ SWI 0xd - clock speed
@********************************************************************

@ int GpClockSpeedChange(int master_speed, int div_factor, int clk_mode);

		.TEXT
                .CODE 32
                .align 4

                .global GpClockSpeedChange
GpClockSpeedChange:
                STMDB    r13!,{r0-r2,r14}
                MOV      r0,r13
                SWI      0xd
                ADD      r13,r13,#0xc
                LDMIA    r13!,{pc}

@********************************************************************
@ SWI 0xf - environment
@********************************************************************

@ int GpAppPathSet(const char * p_path, int n_len);

                .global GpAppPathSet
GpAppPathSet:
                STMDB    r13!,{r2-r4,r14}
                MOV      r4,#0
                SWI      0xf
                LDMIA    r13!,{r2-r4,pc}

@ char * GpAppPathGet(int * n_len /* optional */);

                .global GpAppPathGet
GpAppPathGet:
                STMDB    r13!,{r1-r4,r14}
                MOV      r4,#1
                SWI      0xf
                LDMIA    r13!,{r1-r4,pc}

@  int GpArgSet(int len, char * p_arg);

                .global GpArgSet
GpArgSet:
                STMDB    r13!,{r2-r4,r14}
                MOV      r4,#2
                SWI      0xf
                LDMIA    r13!,{r2-r4,pc}

@ int GpUserInfoGet(char * p_id, char * p_pwd);

                .global GpUserInfoGet
GpUserInfoGet:
                STMDB    r13!,{r2-r6,r14}
                MOV      r5,r0
                MOV      r6,r1
                STMDB    r13!,{r5,r6}
                MOV      r4,#3
                SWI      0xf
                LDMIA    r13!,{r5,r6}
                STMDB    r13!,{r0,r12}
                MOV      r0,r5
                MOV      r1,r6
                @BL       __decode_user_info
                LDMIA    r13!,{r0,r12}
                LDMIA    r13!,{r2-r4,pc}

@********************************************************************
@ SWI 0x10 - Keys
@********************************************************************

		.TEXT
                .CODE 32
                .align 4

                .global _reg_io_key_a
                .global _reg_io_key_b

_reg_io_key_a:  .word 0x00000000
_reg_io_key_b:  .word 0x00000000

                .GLOBAL GpKeyInit
GpKeyInit:
                STMDB sp!,{r1,r2,r3,lr}
                MOV      r0,#0
                SWI      0x10
                adr      r2,_reg_io_key_a
                adr      r3,_reg_io_key_b
                STR      r0,[r2,#0]
                STR      r1,[r3,#0]
                LDMIA sp!,{r1,r2,r3,pc}

@ reports keypad status
@ int GpKeyGet()

		.TEXT
                .CODE 32
                .align 4

                .GLOBAL GpKeyGet
GpKeyGet:

                stmdb   sp!, {r1, r2, r3, r4, r5, lr}
@               mov     r0, #0

                LDR      r3,_reg_io_key_a
                LDR      r4,_reg_io_key_b

                mov     r1, #0
                mvn     r1, r1
                mov     r2, r1

                ldr     r5, [r3]
                and     r1, r1, r5
                ldr     r5, [r4]
                and     r2, r2, r5

                mov r0, #0x01             @ sub function
                swi 0x10                  @

                mvn     r0, r0
                ldmia   sp!, {r1, r2, r3, r4, r5, pc}

@********************************************************************
@ SWI 0x13 - Timer
@********************************************************************

                .global swi_timer_operate
swi_timer_operate:
                STMDB    r13!,{r14}
                ADD      r0,r13,#4
                SWI      0x13
                LDMIA    r13!,{pc}

@********************************************************************
@ SWI 0x14 - IIS Bus
@********************************************************************

                .global swi_iis_operate
swi_iis_operate:
                STMDB    r13!,{r14}
                ADD      r0,r13,#4
                SWI      0x14
                LDMIA    r13!,{pc}

@********************************************************************
@ SWI 0x17 - L3 Control ?
@********************************************************************

@                .global GpControlVolume
@GpControlVolume:
@                STMDB    r13!,{r14}
@                SWI      0x17
@                LDMIA    r13!,{r14,pc}
